@@ -6,18 +6,16 @@ Author: Josef Perktold
 License: BSD-3
 
 """
-
-from statsmodels.compat.python import StringIO
+from io import StringIO
 
 import numpy as np
 from numpy.testing import assert_allclose, assert_equal, assert_
-from nose import SkipTest
 
 import pandas as pd
 import patsy
+import pytest
 
 from statsmodels.discrete.discrete_model import Poisson
-from statsmodels.discrete.discrete_model import Logit
 from statsmodels.genmod.generalized_linear_model import GLM
 from statsmodels.genmod import families
 from statsmodels.base._constraints import fit_constrained
@@ -25,16 +23,17 @@ from statsmodels.base._constraints import fit_constrained
 from statsmodels.tools.tools import add_constant
 from statsmodels import datasets
 
-
-spector_data = datasets.spector.load()
-spector_data.exog = add_constant(spector_data.exog, prepend=False)
-
 from .results import results_poisson_constrained as results
 from .results import results_glm_logit_constrained as reslogit
 
+
+spector_data = datasets.spector.load(as_pandas=False)
+spector_data.exog = add_constant(spector_data.exog, prepend=False)
+
+
 DEBUG = False
 
-ss='''\
+ss = '''\
 agecat	smokes	deaths	pyears
 1	1	32	52407
 2	1	104	43248
@@ -48,6 +47,7 @@ agecat	smokes	deaths	pyears
 5	0	31	1462'''
 
 data = pd.read_csv(StringIO(ss), delimiter='\t')
+data = data.astype(int)
 data['logpyears'] = np.log(data['pyears'])
 
 
@@ -62,20 +62,21 @@ class CheckPoissonConstrainedMixin(object):
         mask = (bse1 == 0) & np.isnan(res2.bse[self.idx])
         assert_allclose(bse1[~mask], res2.bse[self.idx][~mask], rtol=1e-6)
 
-
     def test_basic_method(self):
         if hasattr(self, 'res1m'):
             res1 = (self.res1m if not hasattr(self.res1m, '_results')
-                               else self.res1m._results)
+                    else self.res1m._results)
             res2 = self.res2
             assert_allclose(res1.params, res2.params[self.idx], rtol=1e-6)
 
             # when a parameter is fixed, the Stata has bse=nan, we have bse=0
             mask = (res1.bse == 0) & np.isnan(res2.bse[self.idx])
-            assert_allclose(res1.bse[~mask], res2.bse[self.idx][~mask], rtol=1e-6)
+            assert_allclose(res1.bse[~mask], res2.bse[self.idx][~mask],
+                            rtol=1e-6)
 
             tvalues = res2.params_table[self.idx, 2]
-            # when a parameter is fixed, the Stata has tvalue=nan, we have tvalue=inf
+            # when a parameter is fixed, the Stata has tvalue=nan,
+            #  we have tvalue=inf
             mask = np.isinf(res1.tvalues) & np.isnan(tvalues)
             assert_allclose(res1.tvalues[~mask], tvalues[~mask], rtol=1e-6)
             pvalues = res2.params_table[self.idx, 3]
@@ -91,16 +92,17 @@ class CheckPoissonConstrainedMixin(object):
             # note most pvalues are very small
             # examples so far agree at 8 or more decimal, but rtol is stricter
             # see above: nan versus value
-            assert_allclose(res1.conf_int()[~np.isnan(ci)], ci[~np.isnan(ci)], rtol=5e-5)
+            assert_allclose(res1.conf_int()[~np.isnan(ci)], ci[~np.isnan(ci)],
+                            rtol=5e-5)
 
-            #other
+            # other
             assert_allclose(res1.llf, res2.ll, rtol=1e-6)
             assert_equal(res1.df_model, res2.df_m)
-            # Stata doesn't have df_resid
+            # Stata does not have df_resid
             df_r = res2.N - res2.df_m - 1
             assert_equal(res1.df_resid, df_r)
         else:
-            raise SkipTest("not available yet")
+            pytest.skip("not available yet")
 
     def test_other(self):
         # some results may not be valid or available for all models
@@ -114,11 +116,11 @@ class CheckPoissonConstrainedMixin(object):
                 if DEBUG:
                     import warnings
                     message = ('test: ll_0 not available, llnull=%6.4F'
-                                                                % res1.llnull)
+                               % res1.llnull)
                     warnings.warn(message)
 
         else:
-            raise SkipTest("not available yet")
+            pytest.skip("not available yet")
 
 
 class TestPoissonConstrained1a(CheckPoissonConstrainedMixin):
@@ -127,37 +129,42 @@ class TestPoissonConstrained1a(CheckPoissonConstrainedMixin):
     def setup_class(cls):
 
         cls.res2 = results.results_noexposure_constraint
-        cls.idx = [7, 3, 4, 5, 6, 0, 1]  # 2 is dropped baseline for categorical
+        # 2 is dropped baseline for categorical
+        cls.idx = [7, 3, 4, 5, 6, 0, 1]
 
         # example without offset
         formula = 'deaths ~ logpyears + smokes + C(agecat)'
         mod = Poisson.from_formula(formula, data=data)
-        #res1a = mod1a.fit()
         # get start_params, example fails to converge on one py TravisCI
         k_vars = len(mod.exog_names)
         start_params = np.zeros(k_vars)
         start_params[0] = np.log(mod.endog.mean())
         # if we need it, this is desired params
-        p = np.array([-3.93478643,  1.37276214,  2.33077032,  2.71338891,
-                      2.71338891, 0.57966535,  0.97254074])
+        #  p = np.array([-3.93478643,  1.37276214,  2.33077032,  2.71338891,
+        #                2.71338891, 0.57966535,  0.97254074])
 
         constr = 'C(agecat)[T.4] = C(agecat)[T.5]'
         lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constr)
         cls.res1 = fit_constrained(mod, lc.coefs, lc.constants,
-                                        start_params=start_params,
-                                        fit_kwds={'method': 'bfgs',
-                                                  'disp': 0})
+                                   start_params=start_params,
+                                   fit_kwds={'method': 'bfgs', 'disp': 0})
         # TODO: Newton fails
 
         # test method of Poisson, not monkey patched
         cls.res1m = mod.fit_constrained(constr, start_params=start_params,
                                         method='bfgs', disp=0)
 
-    def test_smoke(self):
+    @pytest.mark.smoke
+    def test_summary(self):
         # trailing text in summary, assumes it's the first extra string
-        #NOTE: see comment about convergence in llnull for self.res1m
+        # NOTE: see comment about convergence in llnull for self.res1m
         summ = self.res1m.summary()
         assert_('linear equality constraints' in summ.extra_txt)
+
+    @pytest.mark.smoke
+    def test_summary2(self):
+        # trailing text in summary, assumes it's the first extra string
+        # NOTE: see comment about convergence in llnull for self.res1m
         summ = self.res1m.summary2()
         assert_('linear equality constraints' in summ.extra_txt[0])
 
@@ -168,15 +175,12 @@ class TestPoissonConstrained1b(CheckPoissonConstrainedMixin):
     def setup_class(cls):
 
         cls.res2 = results.results_exposure_constraint
-        #cls.idx = [3, 4, 5, 6, 0, 1]  # 2 is dropped baseline for categorical
         cls.idx = [6, 2, 3, 4, 5, 0]  # 2 is dropped baseline for categorical
 
         # example without offset
         formula = 'deaths ~ smokes + C(agecat)'
         mod = Poisson.from_formula(formula, data=data,
                                    exposure=data['pyears'].values)
-                                   #offset=np.log(data['pyears'].values))
-        #res1a = mod1a.fit()
         constr = 'C(agecat)[T.4] = C(agecat)[T.5]'
         lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constr)
         cls.res1 = fit_constrained(mod, lc.coefs, lc.constants,
@@ -195,14 +199,12 @@ class TestPoissonConstrained1c(CheckPoissonConstrainedMixin):
     def setup_class(cls):
 
         cls.res2 = results.results_exposure_constraint
-        #cls.idx = [3, 4, 5, 6, 0, 1]  # 2 is dropped baseline for categorical
         cls.idx = [6, 2, 3, 4, 5, 0]  # 2 is dropped baseline for categorical
 
         # example without offset
         formula = 'deaths ~ smokes + C(agecat)'
         mod = Poisson.from_formula(formula, data=data,
                                    offset=np.log(data['pyears'].values))
-        #res1a = mod1a.fit()
         constr = 'C(agecat)[T.4] = C(agecat)[T.5]'
         lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constr)
         cls.res1 = fit_constrained(mod, lc.coefs, lc.constants,
@@ -221,12 +223,11 @@ class TestPoissonNoConstrained(CheckPoissonConstrainedMixin):
     def setup_class(cls):
 
         cls.res2 = results.results_exposure_noconstraint
-        cls.idx = [6, 2, 3, 4, 5, 0] # 1 is dropped baseline for categorical
+        cls.idx = [6, 2, 3, 4, 5, 0]  # 1 is dropped baseline for categorical
 
         # example without offset
         formula = 'deaths ~ smokes + C(agecat)'
         mod = Poisson.from_formula(formula, data=data,
-                                   #exposure=data['pyears'].values)
                                    offset=np.log(data['pyears'].values))
         res1 = mod.fit(disp=0)._results
         # res1 is duplicate check, so we can follow the same pattern
@@ -240,7 +241,8 @@ class TestPoissonConstrained2a(CheckPoissonConstrainedMixin):
     def setup_class(cls):
 
         cls.res2 = results.results_noexposure_constraint2
-        cls.idx = [7, 3, 4, 5, 6, 0, 1]  # 2 is dropped baseline for categorical
+        # 2 is dropped baseline for categorical
+        cls.idx = [7, 3, 4, 5, 6, 0, 1]
 
         # example without offset
         formula = 'deaths ~ logpyears + smokes + C(agecat)'
@@ -251,8 +253,8 @@ class TestPoissonConstrained2a(CheckPoissonConstrainedMixin):
         start_params = np.zeros(k_vars)
         start_params[0] = np.log(mod.endog.mean())
         # if we need it, this is desired params
-        p = np.array([-9.43762015,  1.52762442,  2.74155711,  3.58730007,
-                      4.08730007,  1.15987869,  0.12111539])
+        #  p = np.array([-9.43762015,  1.52762442,  2.74155711,  3.58730007,
+        #                4.08730007,  1.15987869,  0.12111539])
 
         constr = 'C(agecat)[T.5] - C(agecat)[T.4] = 0.5'
         lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constr)
@@ -272,15 +274,12 @@ class TestPoissonConstrained2b(CheckPoissonConstrainedMixin):
     def setup_class(cls):
 
         cls.res2 = results.results_exposure_constraint2
-        #cls.idx = [3, 4, 5, 6, 0, 1]  # 2 is dropped baseline for categorical
         cls.idx = [6, 2, 3, 4, 5, 0]  # 2 is dropped baseline for categorical
 
         # example without offset
         formula = 'deaths ~ smokes + C(agecat)'
         mod = Poisson.from_formula(formula, data=data,
                                    exposure=data['pyears'].values)
-                                   #offset=np.log(data['pyears'].values))
-        #res1a = mod1a.fit()
         constr = 'C(agecat)[T.5] - C(agecat)[T.4] = 0.5'
         lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constr)
         cls.res1 = fit_constrained(mod, lc.coefs, lc.constants,
@@ -300,7 +299,6 @@ class TestPoissonConstrained2c(CheckPoissonConstrainedMixin):
     def setup_class(cls):
 
         cls.res2 = results.results_exposure_constraint2
-        #cls.idx = [3, 4, 5, 6, 0, 1]  # 2 is dropped baseline for categorical
         cls.idx = [6, 2, 3, 4, 5, 0]  # 2 is dropped baseline for categorical
 
         # example without offset
@@ -311,8 +309,7 @@ class TestPoissonConstrained2c(CheckPoissonConstrainedMixin):
         constr = 'C(agecat)[T.5] - C(agecat)[T.4] = 0.5'
         lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constr)
         cls.res1 = fit_constrained(mod, lc.coefs, lc.constants,
-                                       fit_kwds={'method':'newton',
-                                                 'disp': 0})
+                                   fit_kwds={'method': 'newton', 'disp': 0})
         cls.constraints = lc
         # TODO: bfgs fails
 
@@ -329,12 +326,13 @@ class TestGLMPoissonConstrained1a(CheckPoissonConstrainedMixin):
         from statsmodels.base._constraints import fit_constrained
 
         cls.res2 = results.results_noexposure_constraint
-        cls.idx = [7, 3, 4, 5, 6, 0, 1]  # 2 is dropped baseline for categorical
+        # 2 is dropped baseline for categorical
+        cls.idx = [7, 3, 4, 5, 6, 0, 1]
 
         # example without offset
         formula = 'deaths ~ logpyears + smokes + C(agecat)'
         mod = GLM.from_formula(formula, data=data,
-                                    family=families.Poisson())
+                               family=families.Poisson())
 
         constr = 'C(agecat)[T.4] = C(agecat)[T.5]'
         lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constr)
@@ -376,7 +374,6 @@ class TestGLMPoissonConstrained1b(CheckPoissonConstrainedMixin):
         formula = 'deaths ~ smokes + C(agecat)'
         mod = Poisson.from_formula(formula, data=data,
                                    exposure=data['pyears'].values)
-                                   #offset=np.log(data['pyears'].values))
 
         constr = 'C(agecat)[T.4] = C(agecat)[T.5]'
         res2 = mod.fit_constrained(constr, start_params=self.res1m.params,
@@ -406,18 +403,21 @@ class CheckGLMConstrainedMixin(CheckPoissonConstrainedMixin):
         res2 = self.res2  # reference results
         res1 = self.res1m
 
-        #assert_allclose(res1.aic, res2.aic, rtol=1e-10)  # far away
+        # FIXME: dont leave commented-out
+        # assert_allclose(res1.aic, res2.aic, rtol=1e-10)  # far away
         # Stata aic in ereturn and in estat ic are very different
         # we have the same as estat ic
-        # see issue #1733
+        # see issue GH#1733
         assert_allclose(res1.aic, res2.infocrit[4], rtol=1e-10)
 
         assert_allclose(res1.bic, res2.bic, rtol=1e-10)
         # bic is deviance based
-        #assert_allclose(res1.bic, res2.infocrit[5], rtol=1e-10)
+        # FIXME: dont leave commented-out
+        #  assert_allclose(res1.bic, res2.infocrit[5], rtol=1e-10)
         assert_allclose(res1.deviance, res2.deviance, rtol=1e-10)
+        # FIXME: dont leave commented-out
         # TODO: which chi2 are these
-        #assert_allclose(res1.pearson_chi2, res2.chi2, rtol=1e-10)
+        # assert_allclose(res1.pearson_chi2, res2.chi2, rtol=1e-10)
 
 
 class TestGLMLogitConstrained1(CheckGLMConstrainedMixin):
@@ -428,7 +428,6 @@ class TestGLMLogitConstrained1(CheckGLMConstrainedMixin):
         # params sequence same as Stata, but Stata reports param = nan
         # and we have param = value = 0
 
-        #res1ul = Logit(data.endog, data.exog).fit(method="newton", disp=0)
         cls.res2 = reslogit.results_constraint1
 
         mod1 = GLM(spector_data.endog, spector_data.exog,
@@ -446,7 +445,6 @@ class TestGLMLogitConstrained2(CheckGLMConstrainedMixin):
     @classmethod
     def setup_class(cls):
         cls.idx = slice(None)  # params sequence same as Stata
-        #res1ul = Logit(data.endog, data.exog).fit(method="newton", disp=0)
         cls.res2 = reslogit.results_constraint2
 
         mod1 = GLM(spector_data.endog, spector_data.exog,
@@ -459,7 +457,6 @@ class TestGLMLogitConstrained2(CheckGLMConstrainedMixin):
         cls.res1 = fit_constrained(mod1, R, q, fit_kwds={'atol': 1e-10})
         cls.constraints_rq = (R, q)
 
-
     def test_predict(self):
         # results only available for this case
         res2 = self.res2  # reference results
@@ -470,15 +467,18 @@ class TestGLMLogitConstrained2(CheckGLMConstrainedMixin):
         assert_allclose(res1.mu, predicted, rtol=1e-10)
         assert_allclose(res1.fittedvalues, predicted, rtol=1e-10)
 
-
-    def test_smoke(self):
+    @pytest.mark.smoke
+    def test_summary(self):
         # trailing text in summary, assumes it's the first extra string
         summ = self.res1m.summary()
         assert_('linear equality constraints' in summ.extra_txt)
+
+    @pytest.mark.smoke
+    def test_summary2(self):
+        # trailing text in summary, assumes it's the first extra string
         summ = self.res1m.summary2()
         assert_('linear equality constraints' in summ.extra_txt[0])
 
-
     def test_fit_constrained_wrap(self):
         # minimal test
         res2 = self.res2  # reference results
@@ -489,32 +489,61 @@ class TestGLMLogitConstrained2(CheckGLMConstrainedMixin):
         assert_allclose(res_wrap.params, res2.params, rtol=1e-6)
 
 
-def junk():
+class TestGLMLogitConstrained2HC(CheckGLMConstrainedMixin):
+
+    @classmethod
+    def setup_class(cls):
+        cls.idx = slice(None)  # params sequence same as Stata
+        cls.res2 = reslogit.results_constraint2_robust
+
+        mod1 = GLM(spector_data.endog, spector_data.exog,
+                   family=families.Binomial())
+
+        # not used to match Stata for HC
+        # nobs, k_params = mod1.exog.shape
+        # k_params -= 1   # one constraint
+        cov_type = 'HC0'
+        cov_kwds = {'scaling_factor': 32/31}
+        # looks like nobs / (nobs - 1) and not (nobs - 1.) / (nobs - k_params)}
+        constr = 'x1 - x3 = 0'
+        cls.res1m = mod1.fit_constrained(constr, cov_type=cov_type,
+                                         cov_kwds=cov_kwds, atol=1e-10)
+
+        R, q = cls.res1m.constraints.coefs, cls.res1m.constraints.constants
+        cls.res1 = fit_constrained(mod1, R, q, fit_kwds={'atol': 1e-10,
+                                                         'cov_type': cov_type,
+                                                         'cov_kwds': cov_kwds})
+        cls.constraints_rq = (R, q)
+
+
+def junk():  # FIXME: make this into a test, or move/remove
     # Singular Matrix in mod1a.fit()
 
-    formula1 = 'deaths ~ smokes + C(agecat)'
+    # same as Stata default
+    formula2 = 'deaths ~ C(agecat) + C(smokes) : C(agecat)'
 
-    formula2 = 'deaths ~ C(agecat) + C(smokes) : C(agecat)'  # same as Stata default
+    mod = Poisson.from_formula(formula2, data=data,
+                               exposure=data['pyears'].values)
 
-    mod = Poisson.from_formula(formula2, data=data, exposure=data['pyears'].values)
+    mod.fit()
 
-    res0 = mod.fit()
-
-    constraints = 'C(smokes)[T.1]:C(agecat)[3] = C(smokes)[T.1]:C(agecat)[4]'
+    constraints = 'C(smokes)[T.1]:C(agecat)[3] = C(smokes)[T.1]:C(agec`at)[4]'
 
     import patsy
     lc = patsy.DesignInfo(mod.exog_names).linear_constraint(constraints)
     R, q = lc.coefs, lc.constants
 
-    resc = mod.fit_constrained(R,q, fit_kwds={'method':'bfgs'})
+    mod.fit_constrained(R, q, fit_kwds={'method': 'bfgs'})
 
     # example without offset
     formula1a = 'deaths ~ logpyears + smokes + C(agecat)'
     mod1a = Poisson.from_formula(formula1a, data=data)
     print(mod1a.exog.shape)
 
-    res1a = mod1a.fit()
-    lc_1a = patsy.DesignInfo(mod1a.exog_names).linear_constraint('C(agecat)[T.4] = C(agecat)[T.5]')
-    resc1a = mod1a.fit_constrained(lc_1a.coefs, lc_1a.constants, fit_kwds={'method':'newton'})
+    mod1a.fit()
+    lc_1a = patsy.DesignInfo(mod1a.exog_names).linear_constraint(
+        'C(agecat)[T.4] = C(agecat)[T.5]')
+    resc1a = mod1a.fit_constrained(lc_1a.coefs, lc_1a.constants,
+                                   fit_kwds={'method': 'newton'})
     print(resc1a[0])
     print(resc1a[1])
