@@ -1,9 +1,3 @@
-import numpy as np
-from statsmodels.base import model
-import statsmodels.base.model as base
-from statsmodels.tools.decorators import cache_readonly
-from scipy.optimize import brent
-
 """
 Implementation of proportional hazards regression models for duration
 data that may be censored ("Cox models").
@@ -20,6 +14,68 @@ B Gillespie (2006).  Checking the assumptions in the Cox proportional
 hazards model.
 http://www.mwsug.org/proceedings/2006/stats/MWSUG-2006-SD08.pdf
 """
+import numpy as np
+
+from statsmodels.base import model
+import statsmodels.base.model as base
+from statsmodels.tools.decorators import cache_readonly
+from statsmodels.compat.pandas import Appender
+
+
+_predict_docstring = """
+    Returns predicted values from the proportional hazards
+    regression model.
+
+    Parameters
+    ----------%(params_doc)s
+    exog : array_like
+        Data to use as `exog` in forming predictions.  If not
+        provided, the `exog` values from the model used to fit the
+        data are used.%(cov_params_doc)s
+    endog : array_like
+        Duration (time) values at which the predictions are made.
+        Only used if pred_type is either 'cumhaz' or 'surv'.  If
+        using model `exog`, defaults to model `endog` (time), but
+        may be provided explicitly to make predictions at
+        alternative times.
+    strata : array_like
+        A vector of stratum values used to form the predictions.
+        Not used (may be 'None') if pred_type is 'lhr' or 'hr'.
+        If `exog` is None, the model stratum values are used.  If
+        `exog` is not None and pred_type is 'surv' or 'cumhaz',
+        stratum values must be provided (unless there is only one
+        stratum).
+    offset : array_like
+        Offset values used to create the predicted values.
+    pred_type : str
+        If 'lhr', returns log hazard ratios, if 'hr' returns
+        hazard ratios, if 'surv' returns the survival function, if
+        'cumhaz' returns the cumulative hazard function.
+
+    Returns
+    -------
+    A bunch containing two fields: `predicted_values` and
+    `standard_errors`.
+
+    Notes
+    -----
+    Standard errors are only returned when predicting the log
+    hazard ratio (pred_type is 'lhr').
+
+    Types `surv` and `cumhaz` require estimation of the cumulative
+    hazard function.
+"""
+
+_predict_params_doc = """
+    params : array_like
+        The proportional hazards model parameters."""
+
+_predict_cov_params_docstring = """
+    cov_params : array_like
+        The covariance matrix of the estimated `params` vector,
+        used to obtain prediction errors if pred_type='lhr',
+        otherwise optional."""
+
 
 
 class PHSurvivalTime(object):
@@ -51,7 +107,7 @@ class PHSurvivalTime(object):
             None, the entry time is treated as being zero, which
             gives no left truncation.  The entry time must be less
             than or equal to `time`.
-        offset : array-like
+        offset : array_like
             An optional array of offsets
         """
 
@@ -81,20 +137,17 @@ class PHSurvivalTime(object):
                              "after event or censoring times")
 
         # Get the row indices for the cases in each stratum
-        if strata is not None:
-            stu = np.unique(strata)
-            #sth = {x: [] for x in stu} # needs >=2.7
-            sth = dict([(x, []) for x in stu])
-            for i,k in enumerate(strata):
-                sth[k].append(i)
-            stratum_rows = [np.asarray(sth[k], dtype=np.int32) for k in stu]
-            stratum_names = stu
-        else:
-            stratum_rows = [np.arange(len(time)),]
-            stratum_names = [0,]
+        stu = np.unique(strata)
+        #sth = {x: [] for x in stu} # needs >=2.7
+        sth = dict([(x, []) for x in stu])
+        for i,k in enumerate(strata):
+            sth[k].append(i)
+        stratum_rows = [np.asarray(sth[k], dtype=np.int32) for k in stu]
+        stratum_names = stu
 
         # Remove strata with no events
         ix = [i for i,ix in enumerate(stratum_rows) if status[ix].sum() > 0]
+        self.nstrat_orig = len(stratum_rows)
         stratum_rows = [stratum_rows[i] for i in ix]
         stratum_names = [stratum_names[i] for i in ix]
 
@@ -200,39 +253,42 @@ class PHSurvivalTime(object):
                 risk_exit1[ix].append(i)
 
             self.ufailt.append(uft)
-            self.ufailt_ix.append([np.asarray(x, dtype=np.int32) for x in uft_ix])
-            self.risk_enter.append([np.asarray(x, dtype=np.int32) for x in risk_enter1])
-            self.risk_exit.append([np.asarray(x, dtype=np.int32) for x in risk_exit1])
-
+            self.ufailt_ix.append([np.asarray(x, dtype=np.int32)
+                                   for x in uft_ix])
+            self.risk_enter.append([np.asarray(x, dtype=np.int32)
+                                    for x in risk_enter1])
+            self.risk_exit.append([np.asarray(x, dtype=np.int32)
+                                   for x in risk_exit1])
 
 
 class PHReg(model.LikelihoodModel):
     """
-    Fit the Cox proportional hazards regression model for right
-    censored data.
+    Cox Proportional Hazards Regression Model
+
+    The Cox PH Model is for right censored data.
 
     Parameters
     ----------
-    endog : array-like
+    endog : array_like
         The observed times (event or censoring)
-    exog : 2D array-like
+    exog : 2D array_like
         The covariates or exogeneous variables
-    status : array-like
+    status : array_like
         The censoring status values; status=1 indicates that an
-        event occured (e.g. failure or death), status=0 indicates
+        event occurred (e.g. failure or death), status=0 indicates
         that the observation was right censored. If None, defaults
         to status=1 for all cases.
-    entry : array-like
+    entry : array_like
         The entry times, if left truncation occurs
-    strata : array-like
+    strata : array_like
         Stratum labels.  If None, all observations are taken to be
         in a single stratum.
-    ties : string
+    ties : str
         The method used to handle tied times, must be either 'breslow'
         or 'efron'.
-    offset : array-like
+    offset : array_like
         Array of offset values
-    missing : string
+    missing : str
         The method used to handle missing data
 
     Notes
@@ -272,10 +328,16 @@ class PHReg(model.LikelihoodModel):
         self.surv = PHSurvivalTime(self.endog, self.status,
                                     self.exog, self.strata,
                                     self.entry, self.offset)
+        self.nobs = len(self.endog)
+        self.groups = None
 
         # TODO: not used?
         self.missing = missing
 
+        self.df_resid = (np.float(self.exog.shape[0] -
+                                  np.linalg.matrix_rank(self.exog)))
+        self.df_model = np.float(np.linalg.matrix_rank(self.exog))
+
         ties = ties.lower()
         if ties not in ("efron", "breslow"):
             raise ValueError("`ties` must be either `efron` or " +
@@ -295,28 +357,28 @@ class PHReg(model.LikelihoodModel):
         ----------
         formula : str or generic Formula object
             The formula specifying the model
-        data : array-like
+        data : array_like
             The data for the model. See Notes.
-        status : array-like
+        status : array_like
             The censoring status values; status=1 indicates that an
-            event occured (e.g. failure or death), status=0 indicates
+            event occurred (e.g. failure or death), status=0 indicates
             that the observation was right censored. If None, defaults
             to status=1 for all cases.
-        entry : array-like
+        entry : array_like
             The entry times, if left truncation occurs
-        strata : array-like
+        strata : array_like
             Stratum labels.  If None, all observations are taken to be
             in a single stratum.
-        offset : array-like
+        offset : array_like
             Array of offset values
-        subset : array-like
+        subset : array_like
             An array-like object of booleans, integers, or index
             values that indicate the subset of df to use in the
             model. Assumes df is a `pandas.DataFrame`
-        ties : string
+        ties : str
             The method used to handle tied times, must be either 'breslow'
             or 'efron'.
-        missing : string
+        missing : str
             The method used to handle missing data
         args : extra arguments
             These are passed to the model
@@ -334,19 +396,28 @@ class PHReg(model.LikelihoodModel):
         """
 
         # Allow array arguments to be passed by column name.
-        if type(status) is str:
+        if isinstance(status, str):
             status = data[status]
-        if type(entry) is str:
+        if isinstance(entry, str):
             entry = data[entry]
-        if type(strata) is str:
+        if isinstance(strata, str):
             strata = data[strata]
-        if type(offset) is str:
+        if isinstance(offset, str):
             offset = data[offset]
 
+        import re
+        terms = re.split(r"[+\-~]", formula)
+        for term in terms:
+            term = term.strip()
+            if term in ("0", "1"):
+                import warnings
+                warnings.warn("PHReg formulas should not include any '0' or '1' terms")
+
         mod = super(PHReg, cls).from_formula(formula, data,
                     status=status, entry=entry, strata=strata,
                     offset=offset, subset=subset, ties=ties,
-                    missing=missing, *args, **kwargs)
+                    missing=missing, drop_cols=["Intercept"], *args,
+                    **kwargs)
 
         return mod
 
@@ -356,16 +427,23 @@ class PHReg(model.LikelihoodModel):
 
         Parameters
         ----------
-        groups : array-like
+        groups : array_like
             Labels indicating groups of observations that may be
             dependent.  If present, the standard errors account for
             this dependence. Does not affect fitted values.
 
-        Returns a PHregResults instance.
+        Returns
+        -------
+        PHRegResults
+            Returns a results instance.
         """
 
         # TODO process for missing values
         if groups is not None:
+            if len(groups) != len(self.endog):
+                msg = ("len(groups) = %d and len(endog) = %d differ" %
+                       (len(groups), len(self.endog)))
+                raise ValueError(msg)
             self.groups = np.asarray(groups)
         else:
             self.groups = None
@@ -383,171 +461,78 @@ class PHReg(model.LikelihoodModel):
 
         return results
 
-
-    def fit_regularized(self, method="coord_descent", maxiter=100,
-                        alpha=0., L1_wt=1., start_params=None,
-                        cnvrg_tol=1e-7, zero_tol=1e-8, **kwargs):
-        """
+    def fit_regularized(self, method="elastic_net", alpha=0.,
+                        start_params=None, refit=False, **kwargs):
+        r"""
         Return a regularized fit to a linear regression model.
 
         Parameters
         ----------
-        method :
-            Only the coordinate descent algorithm is implemented.
-        maxiter : integer
-            The maximum number of iteration cycles (an iteration cycle
-            involves running coordinate descent on all variables).
-        alpha : scalar or array-like
+        method : {'elastic_net'}
+            Only the `elastic_net` approach is currently implemented.
+        alpha : scalar or array_like
             The penalty weight.  If a scalar, the same penalty weight
             applies to all variables in the model.  If a vector, it
             must have the same length as `params`, and contains a
             penalty weight for each coefficient.
-        L1_wt : scalar
-            The fraction of the penalty given to the L1 penalty term.
-            Must be between 0 and 1 (inclusive).  If 0, the fit is
-            a ridge fit, if 1 it is a lasso fit.
-        start_params : array-like
+        start_params : array_like
             Starting values for `params`.
-        cnvrg_tol : scalar
-            If `params` changes by less than this amount (in sup-norm)
-            in once iteration cycle, the algorithm terminates with
-            convergence.
-        zero_tol : scalar
-            Any estimated coefficient smaller than this value is
-            replaced with zero.
+        refit : bool
+            If True, the model is refit using only the variables that
+            have non-zero coefficients in the regularized fit.  The
+            refitted model is not regularized.
+        **kwargs
+            Additional keyword arguments used to fit the model.
 
         Returns
         -------
-        A PHregResults object, of the same type returned by `fit`.
+        PHRegResults
+            Returns a results instance.
 
         Notes
         -----
-        The penalty is the"elastic net" penalty, which
-        is a convex combination of L1 and L2 penalties.
+        The penalty is the ``elastic net`` penalty, which is a
+        combination of L1 and L2 penalties.
 
-        The function that is minimized is: ..math::
+        The function that is minimized is:
 
-            -loglike/n + alpha*((1-L1_wt)*|params|_2^2/2 + L1_wt*|params|_1)
+        .. math::
+
+            -loglike/n + alpha*((1-L1\_wt)*|params|_2^2/2 + L1\_wt*|params|_1)
 
         where :math:`|*|_1` and :math:`|*|_2` are the L1 and L2 norms.
 
         Post-estimation results are based on the same data used to
         select variables, hence may be subject to overfitting biases.
+
+        The elastic_net method uses the following keyword arguments:
+
+        maxiter : int
+            Maximum number of iterations
+        L1_wt  : float
+            Must be in [0, 1].  The L1 penalty has weight L1_wt and the
+            L2 penalty has weight 1 - L1_wt.
+        cnvrg_tol : float
+            Convergence threshold for line searches
+        zero_tol : float
+            Coefficients below this threshold are treated as zero.
         """
 
-        k_exog = self.exog.shape[1]
-        n_exog = self.exog.shape[0]
+        from statsmodels.base.elastic_net import fit_elasticnet
 
-        if np.isscalar(alpha):
-            alpha = alpha * np.ones(k_exog, dtype=np.float64)
+        if method != "elastic_net":
+            raise ValueError("method for fit_regularied must be elastic_net")
 
-        # regularization cannot be used with groups
-        self.groups = None
+        defaults = {"maxiter" : 50, "L1_wt" : 1, "cnvrg_tol" : 1e-10,
+                    "zero_tol" : 1e-10}
+        defaults.update(kwargs)
 
-        # Define starting params
-        if start_params is None:
-            params = np.zeros(k_exog, dtype=np.float64)
-        else:
-            params = start_params.copy()
+        return fit_elasticnet(self, method=method,
+                              alpha=alpha,
+                              start_params=start_params,
+                              refit=refit,
+                              **defaults)
 
-        # Maybe could be a shallow copy, but just in case...
-        import copy
-        surv = copy.deepcopy(self.surv)
-
-        # This is the base offset, onto which the effects of
-        # constrained variables are added.
-        if self.offset is None:
-            offset_s_base = [np.zeros(len(x)) for x in surv.stratum_rows]
-            surv.offset_s = [x.copy() for x in offset_s_base]
-        else:
-            offset_s_base = [x.copy() for x in surv.offset_s]
-
-        # Create a model instance for optimizing a single variable
-        model_1var = copy.deepcopy(self)
-        model_1var.surv = surv
-        model_1var.ties = self.ties
-
-        # All the negative penalized loglikeihood functions.
-        def gen_npfuncs(k):
-            def nploglike(params):
-                pen = alpha[k]*((1 - L1_wt)*params**2/2 + L1_wt*np.abs(params))
-                return -model_1var.loglike(np.r_[params]) / n_exog + pen
-            def npscore(params):
-                pen_grad = alpha[k]*(1 - L1_wt)*params
-                return -model_1var.score(np.r_[params])[0] / n_exog + pen_grad
-            def nphess(params):
-                pen_hess = alpha[k]*(1 - L1_wt)
-                return -model_1var.hessian(np.r_[params])[0,0] / n_exog + pen_hess
-            return nploglike, npscore, nphess
-        nploglike_funcs = [gen_npfuncs(k) for k in range(len(params))]
-
-        # 1-dimensional exog's
-        exog_s = []
-        for k in range(k_exog):
-            ex = [x[:, k][:, None] for x in surv.exog_s]
-            exog_s.append(ex)
-
-        converged = False
-        btol = 1e-8
-        params_zero = np.zeros(len(params), dtype=bool)
-
-        for itr in range(maxiter):
-
-            # Sweep through the parameters
-            params_save = params.copy()
-            for k in range(k_exog):
-
-                # Under the active set method, if a parameter becomes
-                # zero we don't try to change it again.
-                if params_zero[k]:
-                    continue
-
-                # Set exog to include only the variable whose effect
-                # is being estimated.
-                surv.exog_s = exog_s[k]
-
-                # Set the offset to account for the variables that are
-                # being held fixed.
-                params0 = params.copy()
-                params0[k] = 0
-                for stx in range(self.surv.nstrat):
-                    v = np.dot(self.surv.exog_s[stx], params0)
-                    surv.offset_s[stx] = offset_s_base[stx] + v
-
-                params[k] = _opt_1d(nploglike_funcs[k], params[k],
-                                    alpha[k]*L1_wt, tol=btol)
-
-                # Update the active set
-                if itr > 0 and np.abs(params[k]) < zero_tol:
-                    params_zero[k] = True
-                    params[k] = 0.
-
-            # Check for convergence
-            pchange = np.max(np.abs(params - params_save))
-            if pchange < cnvrg_tol:
-                converged = True
-                break
-
-        # Set approximate zero coefficients to be exactly zero
-        params *= np.abs(params) >= zero_tol
-
-        # Fit the reduced model to get standard errors and other
-        # post-estimation results.
-        ii = np.flatnonzero(params)
-        cov = np.zeros((k_exog, k_exog), dtype=np.float64)
-        if len(ii) > 0:
-            model = self.__class__(self.endog, self.exog[:, ii],
-                                   status=self.status, entry=self.entry,
-                                   strata=self.strata, offset=self.offset,
-                                   ties=self.ties, missing=self.missing)
-            rslt = model.fit()
-            cov[np.ix_(ii, ii)] = rslt.normalized_cov_params
-
-        rfit = PHRegResults(self, params, cov_params=cov)
-        rfit.converged = converged
-        rfit.regularized = True
-
-        return rfit
 
     def loglike(self, params):
         """
@@ -823,9 +808,8 @@ class PHReg(model.LikelihoodModel):
                     xp0 += e_linpred[ix].sum()
                     v = exog_s[ix,:]
                     xp1 += (e_linpred[ix][:,None] * v).sum(0)
-                    mat = v[None,:,:]
                     elx = e_linpred[ix]
-                    xp2 += (mat.T * mat * elx[None,:,None]).sum(1)
+                    xp2 += np.einsum("ij,ik,i->jk", v, v, elx)
 
                 # Account for all cases that fail at this point.
                 m = len(uft_ix[i])
@@ -837,10 +821,8 @@ class PHReg(model.LikelihoodModel):
                     xp0 -= e_linpred[ix].sum()
                     v = exog_s[ix,:]
                     xp1 -= (e_linpred[ix][:,None] * v).sum(0)
-                    mat = v[None,:,:]
                     elx = e_linpred[ix]
-                    xp2 -= (mat.T * mat * elx[None,:,None]).sum(1)
-
+                    xp2 -= np.einsum("ij,ik,i->jk", v, v, elx)
         return -hess
 
     def efron_hessian(self, params):
@@ -878,28 +860,25 @@ class PHReg(model.LikelihoodModel):
                     xp0 += e_linpred[ix].sum()
                     v = exog_s[ix,:]
                     xp1 += (e_linpred[ix][:,None] * v).sum(0)
-                    mat = v[None,:,:]
                     elx = e_linpred[ix]
-                    xp2 += (mat.T * mat * elx[None,:,None]).sum(1)
+                    xp2 += np.einsum("ij,ik,i->jk", v, v, elx)
 
                 ixf = uft_ix[i]
                 if len(ixf) > 0:
                     v = exog_s[ixf,:]
                     xp0f = e_linpred[ixf].sum()
                     xp1f = (e_linpred[ixf][:,None] * v).sum(0)
-                    mat = v[None,:,:]
                     elx = e_linpred[ixf]
-                    xp2f = (mat.T * mat * elx[None,:,None]).sum(1)
+                    xp2f = np.einsum("ij,ik,i->jk", v, v, elx)
 
                 # Account for all cases that fail at this point.
                 m = len(uft_ix[i])
                 J = np.arange(m, dtype=np.float64) / m
                 c0 = xp0 - J*xp0f
-                mat = (xp2[None,:,:] - J[:,None,None]*xp2f) / c0[:,None,None]
-                hess += mat.sum(0)
+                hess += xp2 * np.sum(1 / c0)
+                hess -= xp2f * np.sum(J / c0)
                 mat = (xp1[None, :] - np.outer(J, xp1f)) / c0[:, None]
-                mat = mat[:, :, None] * mat[:, None, :]
-                hess -= mat.sum(0)
+                hess -= np.einsum("ij,ik->jk", mat, mat)
 
                 # Update for new cases entering the risk set.
                 ix = surv.risk_exit[stx][i]
@@ -907,9 +886,8 @@ class PHReg(model.LikelihoodModel):
                     xp0 -= e_linpred[ix].sum()
                     v = exog_s[ix,:]
                     xp1 -= (e_linpred[ix][:,None] * v).sum(0)
-                    mat = v[None,:,:]
                     elx = e_linpred[ix]
-                    xp2 -= (mat.T * mat * elx[None,:,None]).sum(1)
+                    xp2 -= np.einsum("ij,ik,i->jk", v, v, elx)
 
         return -hess
 
@@ -1168,8 +1146,8 @@ class PHReg(model.LikelihoodModel):
                 xp0 -= e_linpred[ix].sum()
 
             cumhaz = np.cumsum(h0) - h0
-            surv = np.exp(-cumhaz)
-            rslt.append([uft, cumhaz, surv])
+            current_strata_surv = np.exp(-cumhaz)
+            rslt.append([uft, cumhaz, current_strata_surv])
 
         return rslt
 
@@ -1204,57 +1182,11 @@ class PHReg(model.LikelihoodModel):
 
         return cumhaz_f
 
-    def predict(self, params, cov_params=None, endog=None, exog=None,
+    @Appender(_predict_docstring % {
+        'params_doc': _predict_params_doc,
+        'cov_params_doc': _predict_cov_params_docstring})
+    def predict(self, params, exog=None, cov_params=None, endog=None,
                 strata=None, offset=None, pred_type="lhr"):
-        """
-        Returns predicted values from the proportional hazards
-        regression model.
-
-        Parameters
-        ----------
-        params : array-like
-            The proportional hazards model parameters.
-        cov_params : array-like
-            The covariance matrix of the estimated `params` vector,
-            used to obtain prediction errors if pred_type='lhr',
-            otherwise optional.
-        endog : array-like
-            Duration (time) values at which the predictions are made.
-            Only used if pred_type is either 'cumhaz' or 'surv'.  If
-            using model `exog`, defaults to model `endog` (time), but
-            may be provided explicitly to make predictions at
-            alternative times.
-        exog : array-like
-            Data to use as `exog` in forming predictions.  If not
-            provided, the `exog` values from the model used to fit the
-            data are used.
-        strata : array-like
-            A vector of stratum values used to form the predictions.
-            Not used (may be 'None') if pred_type is 'lhr' or 'hr'.
-            If `exog` is None, the model stratum values are used.  If
-            `exog` is not None and pred_type is 'surv' or 'cumhaz',
-            stratum values must be provided (unless there is only one
-            stratum).
-        offset : array-like
-            Offset values used to create the predicted values.
-        pred_type : string
-            If 'lhr', returns log hazard ratios, if 'hr' returns
-            hazard ratios, if 'surv' returns the survival function, if
-            'cumhaz' returns the cumulative hazard function.
-
-        Returns
-        -------
-        A bunch containing two fields: `predicted_values` and
-        `standard_errors`.
-
-        Notes
-        -----
-        Standard errors are only returned when predicting the log
-        hazard ratio (pred_type is 'lhr').
-
-        Types `surv` and `cumhaz` require estimation of the cumulative
-        hazard function.
-        """
 
         pred_type = pred_type.lower()
         if pred_type not in ["lhr", "hr", "surv", "cumhaz"]:
@@ -1266,7 +1198,7 @@ class PHReg(model.LikelihoodModel):
             standard_errors = None
         ret_val = bunch()
 
-        # Don't do anything with offset here because we want to allow
+        # Do not do anything with offset here because we want to allow
         # different offsets to be specified even if exog is the model
         # exog.
         exog_provided = True
@@ -1281,14 +1213,15 @@ class PHReg(model.LikelihoodModel):
         elif self.offset is not None and not exog_provided:
             lhr += self.offset
 
-        # Handle lhr and hr prediction first, since they don't make
+        # Handle lhr and hr prediction first, since they do not make
         # use of the hazard function.
 
         if pred_type == "lhr":
             ret_val.predicted_values = lhr
-            mat = np.dot(exog, cov_params)
-            va = (mat * exog).sum(1)
-            ret_val.standard_errors = np.sqrt(va)
+            if cov_params is not None:
+                mat = np.dot(exog, cov_params)
+                va = (mat * exog).sum(1)
+                ret_val.standard_errors = np.sqrt(va)
             return ret_val
 
         hr = np.exp(lhr)
@@ -1338,8 +1271,8 @@ class PHReg(model.LikelihoodModel):
 
         Parameters
         ----------
-        params : arrayh-like
-            The model proportional hazards model parameters.
+        params : array_like
+            The proportional hazards model parameters.
 
         Returns
         -------
@@ -1349,15 +1282,13 @@ class PHReg(model.LikelihoodModel):
         -----
         The distributions are obtained from a simple discrete estimate
         of the survivor function that puts all mass on the observed
-        failure times wihtin a stratum.
+        failure times within a stratum.
         """
 
         # TODO: this returns a Python list of rv_discrete objects, so
         # nothing can be vectorized.  It appears that rv_discrete does
         # not allow vectorization.
 
-        from scipy.stats.distributions import rv_discrete
-
         surv = self.surv
         bhaz = self.baseline_cumulative_hazard(params)
 
@@ -1384,8 +1315,8 @@ class PHReg(model.LikelihoodModel):
 
             # The individual survival functions.
             usurv = np.exp(-ichaz)
-            usurv = np.concatenate((usurv, np.zeros((usurv.shape[0], 1))),
-                                   axis=1)
+            z = np.zeros((usurv.shape[0], 1))
+            usurv = np.concatenate((usurv, z), axis=1)
 
             # The individual survival probability masses.
             probs = -np.diff(usurv, 1)
@@ -1397,15 +1328,15 @@ class PHReg(model.LikelihoodModel):
         mxc = max([x.shape[1] for x in xk])
         for k in range(self.surv.nstrat):
             if xk[k].shape[1] < mxc:
-                xk1 = np.zeros((xk.shape[0], mxc))
-                pk1 = np.zeros((pk.shape[0], mxc))
-                xk1[:, -mxc:] = xk
-                pk1[:, -mxc:] = pk
+                xk1 = np.zeros((xk[k].shape[0], mxc))
+                pk1 = np.zeros((pk[k].shape[0], mxc))
+                xk1[:, 0:xk[k].shape[1]] = xk[k]
+                pk1[:, 0:pk[k].shape[1]] = pk[k]
                 xk[k], pk[k] = xk1, pk1
 
-        xka = np.nan * np.zeros((len(self.endog), mxc), dtype=np.float64)
+        # Put the support points and probabilities into single matrices
+        xka = np.nan * np.ones((len(self.endog), mxc))
         pka = np.ones((len(self.endog), mxc), dtype=np.float64) / mxc
-
         for stx in range(self.surv.nstrat):
             ix = self.surv.stratum_rows[stx]
             xka[ix, :] = xk[stx]
@@ -1427,19 +1358,17 @@ class PHRegResults(base.LikelihoodModelResults):
     ----------
     See statsmodels.LikelihoodModelResults
 
-    Returns
-    -------
-    **Attributes**
-
+    Attributes
+    ----------
     model : class instance
         PHreg model instance that called fit.
-    normalized_cov_params : array
+    normalized_cov_params : ndarray
         The sampling covariance matrix of the estimates
-    params : array
+    params : ndarray
         The coefficients of the fitted model.  Each coefficient is the
         log hazard ratio corresponding to a 1 unit difference in a
         single covariate while holding the other covariates fixed.
-    bse : array
+    bse : ndarray
         The standard errors of the fitted parameters.
 
     See Also
@@ -1447,11 +1376,16 @@ class PHRegResults(base.LikelihoodModelResults):
     statsmodels.LikelihoodModelResults
     '''
 
-    def __init__(self, model, params, cov_params, covariance_type="naive"):
+    def __init__(self, model, params, cov_params, scale=1., covariance_type="naive"):
+
+        # There is no scale parameter, but we need it for
+        # meta-procedures that work with results.
 
         self.covariance_type = covariance_type
+        self.df_resid = model.df_resid
+        self.df_model = model.df_model
 
-        super(PHRegResults, self).__init__(model, params,
+        super(PHRegResults, self).__init__(model, params, scale=1.,
            normalized_cov_params=cov_params)
 
     @cache_readonly
@@ -1482,75 +1416,29 @@ class PHRegResults(base.LikelihoodModelResults):
         -----
         The distributions are obtained from a simple discrete estimate
         of the survivor function that puts all mass on the observed
-        failure times wihtin a stratum.
+        failure times within a stratum.
         """
 
         return self.model.get_distribution(self.params)
 
-
+    @Appender(_predict_docstring % {'params_doc': '', 'cov_params_doc': ''})
     def predict(self, endog=None, exog=None, strata=None,
-                offset=None, pred_type="lhr"):
-        """
-        Returns predicted values from the fitted proportional hazards
-        regression model.
-
-        Parameters
-        ----------
-        params : array-;like
-            The proportional hazards model parameters.
-        endog : array-like
-            Duration (time) values at which the predictions are made.
-            Only used if pred_type is either 'cumhaz' or 'surv'.  If
-            using model `exog`, defaults to model `endog` (time), but
-            may be provided explicitly to make predictions at
-            alternative times.
-        exog : array-like
-            Data to use as `exog` in forming predictions.  If not
-            provided, the `exog` values from the model used to fit the
-            data are used.
-        strata : array-like
-            A vector of stratum values used to form the predictions.
-            Not used (may be 'None') if pred_type is 'lhr' or 'hr'.
-            If `exog` is None, the model stratum values are used.  If
-            `exog` is not None and pred_type is 'surv' or 'cumhaz',
-            stratum values must be provided (unless there is only one
-            stratum).
-        offset : array-like
-            Offset values used to create the predicted values.
-        pred_type : string
-            If 'lhr', returns log hazard ratios, if 'hr' returns
-            hazard ratios, if 'surv' returns the survival function, if
-            'cumhaz' returns the cumulative hazard function.
-
-        Returns
-        -------
-        A bunch containing two fields: `predicted_values` and
-        `standard_errors`.
-
-        Notes
-        -----
-        Standard errors are only returned when predicting the log
-        hazard ratio (pred_type is 'lhr').
-
-        Types `surv` and `cumhaz` require estimation of the cumulative
-        hazard function.
-        """
-
-        return self.model.predict(self.params, self.cov_params(),
-                                  endog, exog, strata, offset,
-                                  pred_type)
+                offset=None, transform=True, pred_type="lhr"):
+        return super(PHRegResults, self).predict(exog=exog,
+                                                 transform=transform,
+                                                 cov_params=self.cov_params(),
+                                                 endog=endog,
+                                                 strata=strata,
+                                                 offset=offset,
+                                                 pred_type=pred_type)
 
     def _group_stats(self, groups):
         """
         Descriptive statistics of the groups.
         """
-        gsize = {}
-        for x in groups:
-            if x not in gsize:
-                gsize[x] = 0
-            gsize[x] += 1
-        gsize = np.asarray(gsize.values())
-        return gsize.min(), gsize.max(), gsize.mean()
+        gsizes = np.unique(groups, return_counts=True)
+        gsizes = gsizes[1]
+        return gsizes.min(), gsizes.max(), gsizes.mean(), len(gsizes)
 
     @cache_readonly
     def weighted_covariate_averages(self):
@@ -1660,12 +1548,14 @@ class PHRegResults(base.LikelihoodModelResults):
         Summarize the proportional hazards regression results.
 
         Parameters
-        -----------
-        yname : string, optional
+        ----------
+        yname : str, optional
             Default is `y`
-        xname : list of strings, optional
-            Default is `x#` for ## in p the number of regressors
-        title : string, optional
+        xname : list[str], optional
+            Names for the exogenous variables, default is `x#` for ## in p the
+            number of regressors. Must match the number of parameters in
+            the model
+        title : str, optional
             Title for the top table. If not None, then this replaces
             the default title
         alpha : float
@@ -1679,12 +1569,11 @@ class PHRegResults(base.LikelihoodModelResults):
 
         See Also
         --------
-        statsmodels.iolib.summary.Summary : class to hold summary
-            results
+        statsmodels.iolib.summary2.Summary : class to hold summary results
         """
 
         from statsmodels.iolib import summary2
-        from statsmodels.compat.collections import OrderedDict
+        from collections import OrderedDict
         smry = summary2.Summary()
         float_format = "%8.3f"
 
@@ -1698,10 +1587,18 @@ class PHRegResults(base.LikelihoodModelResults):
         info["Num. events:"] = str(int(sum(self.model.status)))
 
         if self.model.groups is not None:
-            mn, mx, avg = self._group_stats(self.model.groups)
-            info["Max. group size:"] = str(mx)
-            info["Min. group size:"] = str(mn)
-            info["Avg. group size:"] = str(avg)
+            mn, mx, avg, num = self._group_stats(self.model.groups)
+            info["Num groups:"] = "%.0f" % num
+            info["Min group size:"] = "%.0f" % mn
+            info["Max group size:"] = "%.0f" % mx
+            info["Avg group size:"] = "%.1f" % avg
+
+        if self.model.strata is not None:
+            mn, mx, avg, num = self._group_stats(self.model.strata)
+            info["Num strata:"] = "%.0f" % num
+            info["Min stratum size:"] = "%.0f" % mn
+            info["Max stratum size:"] = "%.0f" % mx
+            info["Avg stratum size:"] = "%.1f" % avg
 
         smry.add_dict(info, align='l', float_format=float_format)
 
@@ -1713,12 +1610,26 @@ class PHRegResults(base.LikelihoodModelResults):
         param.loc[:, a] = np.exp(param.loc[:, a])
         a = "%.3f]" % (1 - alpha / 2)
         param.loc[:, a] = np.exp(param.loc[:, a])
-        if xname != None:
+        if xname is not None:
             param.index = xname
         smry.add_df(param, float_format=float_format)
         smry.add_title(title=title, results=self)
         smry.add_text("Confidence intervals are for the hazard ratios")
 
+        dstrat = self.model.surv.nstrat_orig - self.model.surv.nstrat
+        if dstrat > 0:
+            if dstrat == 1:
+                smry.add_text("1 stratum dropped for having no events")
+            else:
+                smry.add_text("%d strata dropped for having no events" % dstrat)
+
+        if self.model.entry is not None:
+            n_entry = sum(self.model.entry != 0)
+            if n_entry == 1:
+                smry.add_text("1 observation has a positive entry time")
+            else:
+                smry.add_text("%d observations have positive entry times" % n_entry)
+
         if self.model.groups is not None:
             smry.add_text("Standard errors account for dependence within groups")
 
@@ -1727,16 +1638,17 @@ class PHRegResults(base.LikelihoodModelResults):
 
         return smry
 
+
 class rv_discrete_float(object):
     """
     A class representing a collection of discrete distributions.
 
     Parameters
     ----------
-    xk : 2d array-like
+    xk : 2d array_like
         The support points, should be non-decreasing within each
         row.
-    pk : 2d array-like
+    pk : 2d array_like
         The probabilities, should sum to one within each row.
 
     Notes
@@ -1814,55 +1726,3 @@ class rv_discrete_float(object):
         """
 
         return np.sqrt(self.var())
-
-
-def _opt_1d(funcs, start, L1_wt, tol):
-    """
-    Optimize a L1-penalized smooth one-dimensional function of a
-    single variable.
-
-    Parameters
-    ----------
-    funcs : tuple of functions
-        funcs[0] is the objective function to be minimized.  funcs[1]
-        and funcs[2] are, respectively, the first and second
-        derivatives of the smooth part of funcs[0] (i.e. excluding the
-        L1 penalty).
-    start : real
-        A starting value for the function argument
-    L1_wt : non-negative real
-        The weight for the L1 penalty function.
-    tol : non-negative real
-        A convergence threshold.
-
-    Returns
-    -------
-    The argmin of the objective function.
-    """
-
-    # TODO: can we detect failures without calling funcs[0] twice?
-
-    x = start
-    f = funcs[0](x)
-
-    b = funcs[1](x)
-    c = funcs[2](x)
-    d = b - c*x
-
-    if L1_wt > np.abs(d):
-        return 0.
-    elif d >= 0:
-        x += (L1_wt - b) / c
-    elif d < 0:
-        x -= (L1_wt + b) / c
-
-    f1 = funcs[0](x)
-
-    # This is an expensive fall-back if the quadratic
-    # approximation is poor and sends us far off-course.
-    if f1 > f + 1e-10:
-        return brent(funcs[0], brack=(x-0.2, x+0.2), tol=tol)
-
-    return x
-
-
