@@ -2,7 +2,7 @@
 """
 Authors:    Josef Perktold, Skipper Seabold, Denis A. Engemann
 """
-from statsmodels.compat.python import iterkeys, lrange, zip, iteritems
+from statsmodels.compat.python import iterkeys, lrange, iteritems
 import numpy as np
 
 from statsmodels.graphics.plottools import rainbow
@@ -17,30 +17,29 @@ def interaction_plot(x, trace, response, func=np.mean, ax=None, plottype='b',
     Interaction plot for factor level statistics.
 
     Note. If categorial factors are supplied levels will be internally
-    recoded to integers. This ensures matplotlib compatiblity.
-
-    uses pandas.DataFrame to calculate an `aggregate` statistic for each
-    level of the factor or group given by `trace`.
+    recoded to integers. This ensures matplotlib compatibility. Uses
+    a DataFrame to calculate an `aggregate` statistic for each level of the
+    factor or group given by `trace`.
 
     Parameters
     ----------
-    x : array-like
+    x : array_like
         The `x` factor levels constitute the x-axis. If a `pandas.Series` is
         given its name will be used in `xlabel` if `xlabel` is None.
-    trace : array-like
+    trace : array_like
         The `trace` factor levels will be drawn as lines in the plot.
         If `trace` is a `pandas.Series` its name will be used as the
         `legendtitle` if `legendtitle` is None.
-    response : array-like
+    response : array_like
         The reponse or dependent variable. If a `pandas.Series` is given
         its name will be used in `ylabel` if `ylabel` is None.
     func : function
         Anything accepted by `pandas.DataFrame.aggregate`. This is applied to
         the response variable grouped by the trace levels.
-    plottype : str {'line', 'scatter', 'both'}, optional
-        The type of plot to return. Can be 'l', 's', or 'b'
     ax : axes, optional
         Matplotlib axes instance
+    plottype : str {'line', 'scatter', 'both'}, optional
+        The type of plot to return. Can be 'l', 's', or 'b'
     xlabel : str, optional
         Label to use for `x`. Default is 'X'. If `x` is a `pandas.Series` it
         will use the series names.
@@ -49,17 +48,21 @@ def interaction_plot(x, trace, response, func=np.mean, ax=None, plottype='b',
         `response` is a `pandas.Series` it will use the series names.
     colors : list, optional
         If given, must have length == number of levels in trace.
+    markers : list, optional
+        If given, must have length == number of levels in trace
     linestyles : list, optional
         If given, must have length == number of levels in trace.
-    markers : list, optional
-        If given, must have length == number of lovels in trace
-    kwargs
+    legendloc : {None, str, int}
+        Location passed to the legend command.
+    legendtitle : {None, str}
+        Title of the legend.
+    **kwargs
         These will be passed to the plot command used either plot or scatter.
         If you want to control the overall plotting options, use kwargs.
 
     Returns
     -------
-    fig : Figure
+    Figure
         The figure given by `ax.figure` or a new instance.
 
     Examples
@@ -159,7 +162,7 @@ def _recode(x, levels):
 
     Parameters
     ----------
-    x : array-like
+    x : array_like
         array like object supporting with numpy array methods of categorially
         coded data.
     levels : dict
@@ -168,7 +171,6 @@ def _recode(x, levels):
     Returns
     -------
     out : instance numpy.ndarray
-
     """
     from pandas import Series
     name = None
