@@ -4,19 +4,19 @@ Recursive least squares model
 Author: Chad Fulton
 License: Simplified-BSD
 """
-from __future__ import division, absolute_import, print_function
-
-from warnings import warn
-from statsmodels.compat.collections import OrderedDict
 
 import numpy as np
 import pandas as pd
-from statsmodels.regression.linear_model import OLS
+
+from statsmodels.compat.pandas import Appender
+
 from statsmodels.tools.data import _is_using_pandas
 from statsmodels.tsa.statespace.mlemodel import (
-    MLEModel, MLEResults, MLEResultsWrapper)
+    MLEModel, MLEResults, MLEResultsWrapper, PredictionResults,
+    PredictionResultsWrapper)
+from statsmodels.tsa.statespace.tools import concat
 from statsmodels.tools.tools import Bunch
-from statsmodels.tools.decorators import cache_readonly, resettable_cache
+from statsmodels.tools.decorators import cache_readonly
 import statsmodels.base.wrapper as wrap
 
 # Columns are alpha = 0.1, 0.05, 0.025, 0.01, 0.005
@@ -37,6 +37,14 @@ class RecursiveLS(MLEModel):
         The observed time-series process :math:`y`
     exog : array_like
         Array of exogenous regressors, shaped nobs x k.
+    constraints : array_like, str, or tuple
+            - array : An r x k array where r is the number of restrictions to
+              test and k is the number of regressors. It is assumed that the
+              linear combination is equal to zero.
+            - str : The full hypotheses to test can be given as a string.
+              See the examples.
+            - tuple : A tuple of arrays in the form (R, q), ``q`` can be
+              either a scalar or a length p row vector.
 
     Notes
     -----
@@ -48,14 +56,14 @@ class RecursiveLS(MLEModel):
 
     References
     ----------
-    .. [1] Durbin, James, and Siem Jan Koopman. 2012.
+    .. [*] Durbin, James, and Siem Jan Koopman. 2012.
        Time Series Analysis by State Space Methods: Second Edition.
        Oxford University Press.
-
     """
-    def __init__(self, endog, exog, **kwargs):
+    def __init__(self, endog, exog, constraints=None, **kwargs):
         # Standardize data
-        if not _is_using_pandas(endog, None):
+        endog_using_pandas = _is_using_pandas(endog, None)
+        if not endog_using_pandas:
             endog = np.asanyarray(endog)
 
         exog_is_using_pandas = _is_using_pandas(exog, None)
@@ -71,32 +79,68 @@ class RecursiveLS(MLEModel):
 
         self.k_exog = exog.shape[1]
 
+        # Handle constraints
+        self.k_constraints = 0
+        self._r_matrix = self._q_matrix = None
+        if constraints is not None:
+            from patsy import DesignInfo
+            from statsmodels.base.data import handle_data
+            data = handle_data(endog, exog, **kwargs)
+            names = data.param_names
+            LC = DesignInfo(names).linear_constraint(constraints)
+            self._r_matrix, self._q_matrix = LC.coefs, LC.constants
+            self.k_constraints = self._r_matrix.shape[0]
+
+            constraint_endog = np.zeros((len(endog), len(self._r_matrix)))
+            if endog_using_pandas:
+                constraint_endog = pd.DataFrame(constraint_endog,
+                                                index=endog.index)
+                endog = concat([endog, constraint_endog], axis=1)
+                endog.values[:, 1:] = self._q_matrix[:, 0]
+            else:
+                endog[:, 1:] = self._q_matrix[:, 0]
+
         # Handle coefficient initialization
-        # By default, do not calculate likelihood while it is controlled by
-        # diffuse initial conditions.
-        kwargs.setdefault('loglikelihood_burn', self.k_exog)
-        kwargs.setdefault('initialization', 'approximate_diffuse')
-        kwargs.setdefault('initial_variance', 1e9)
+        kwargs.setdefault('initialization', 'diffuse')
 
         # Initialize the state space representation
         super(RecursiveLS, self).__init__(
-            endog, k_states=self.k_exog, exog=exog, **kwargs
-        )
+            endog, k_states=self.k_exog, exog=exog, **kwargs)
+
+        # Use univariate filtering by default
+        self.ssm.filter_univariate = True
+
+        # Concentrate the scale out of the likelihood function
+        self.ssm.filter_concentrated = True
 
         # Setup the state space representation
-        self['design'] = self.exog[:, :, None].T
+        self['design'] = np.zeros((self.k_endog, self.k_states, self.nobs))
+        self['design', 0] = self.exog[:, :, None].T
+        if self._r_matrix is not None:
+            self['design', 1:, :] = self._r_matrix[:, :, None]
         self['transition'] = np.eye(self.k_states)
 
         # Notice that the filter output does not depend on the measurement
         # variance, so we set it here to 1
         self['obs_cov', 0, 0] = 1.
+        self['transition'] = np.eye(self.k_states)
+
+        # Linear constraints are technically imposed by adding "fake" endog
+        # variables that are used during filtering, but for all model- and
+        # results-based purposes we want k_endog = 1.
+        if self._r_matrix is not None:
+            self.k_endog = 1
 
     @classmethod
-    def from_formula(cls, formula, data, subset=None):
-        """
-        Not implemented for state space models
-        """
-        return super(MLEModel, cls).from_formula(formula, data, subset)
+    def from_formula(cls, formula, data, subset=None, constraints=None):
+        return super(MLEModel, cls).from_formula(formula, data, subset,
+                                                 constraints=constraints)
+
+    def _validate_can_fix_params(self, param_names):
+        raise ValueError('Linear constraints on coefficients should be given'
+                         ' using the `constraints` argument in constructing.'
+                         ' the model. Other parameter constraints are not'
+                         ' available in the resursive least squares model.')
 
     def fit(self):
         """
@@ -106,17 +150,12 @@ class RecursiveLS(MLEModel):
         -------
         RecursiveLSResults
         """
-        # Get the smoother results with an arbitrary measurement variance
         smoother_results = self.smooth(return_ssm=True)
-        # Compute the MLE of sigma2 (see Harvey, 1989 equation 4.2.5)
-        resid = smoother_results.standardized_forecasts_error[0]
-        sigma2 = (np.inner(resid, resid) /
-                  (self.nobs - self.loglikelihood_burn))
 
-        # Now construct a results class, where the params are the final
-        # estimates of the regression coefficients
-        self['obs_cov', 0, 0] = sigma2
-        return self.smooth()
+        with self.ssm.fixed_scale(smoother_results.scale):
+            res = self.smooth()
+
+        return res
 
     def filter(self, return_ssm=False, **kwargs):
         # Get the state space output
@@ -164,6 +203,11 @@ class RecursiveLS(MLEModel):
 
         return result
 
+    @property
+    def endog_names(self):
+        endog_names = super(RecursiveLS, self).endog_names
+        return endog_names[0] if isinstance(endog_names, list) else endog_names
+
     @property
     def param_names(self):
         return self.exog_names
@@ -184,7 +228,7 @@ class RecursiveLS(MLEModel):
         ----------
         params : array_like
             Array of new parameters.
-        transformed : boolean, optional
+        transformed : bool, optional
             Whether or not `params` is already transformed. If set to False,
             `transform_params` is called. Default is True..
 
@@ -222,14 +266,26 @@ class RecursiveLSResults(MLEResults):
         super(RecursiveLSResults, self).__init__(
             model, params, filter_results, cov_type, **kwargs)
 
-        self.df_resid = np.inf  # attribute required for wald tests
+        # Since we are overriding params with things that are not MLE params,
+        # need to adjust df's
+        q = max(self.loglikelihood_burn, self.k_diffuse_states)
+        self.df_model = q - self.model.k_constraints
+        self.df_resid = self.nobs_effective - self.df_model
 
         # Save _init_kwds
         self._init_kwds = self.model._get_init_kwds()
 
         # Save the model specification
         self.specification = Bunch(**{
-            'k_exog': self.model.k_exog})
+            'k_exog': self.model.k_exog,
+            'k_constraints': self.model.k_constraints})
+
+        # Adjust results to remove "faux" endog from the constraints
+        if self.model._r_matrix is not None:
+            for name in ['forecasts', 'forecasts_error',
+                         'forecasts_error_cov', 'standardized_forecasts_error',
+                         'forecasts_error_diffuse_cov']:
+                setattr(self, name, getattr(self, name)[0:1])
 
     @property
     def recursive_coefficients(self):
@@ -271,7 +327,7 @@ class RecursiveLSResults(MLEResults):
 
     @cache_readonly
     def resid_recursive(self):
-        """
+        r"""
         Recursive residuals
 
         Returns
@@ -282,17 +338,21 @@ class RecursiveLSResults(MLEResults):
 
         Notes
         -----
-        The first `k_exog` residuals are typically unreliable due to
-        initialization.
+        These quantities are defined in, for example, Harvey (1989)
+        section 5.4. In fact, there he defines the standardized innovations in
+        equation 5.4.1, but in his version they have non-unit variance, whereas
+        the standardized forecast errors computed by the Kalman filter here
+        assume unit variance. To convert to Harvey's definition, we need to
+        multiply by the standard deviation.
 
+        Harvey notes that in smaller samples, "although the second moment
+        of the :math:`\tilde \sigma_*^{-1} \tilde v_t`'s is unity, the
+        variance is not necessarily equal to unity as the mean need not be
+        equal to zero", and he defines an alternative version (which are
+        not provided here).
         """
-        # See Harvey (1989) section 5.4; he defines the standardized
-        # innovations in 5.4.1, but they have non-unit variance, whereas
-        # the standardized forecast errors assume unit variance. To convert
-        # to Harvey's definition, we need to multiply by the standard
-        # deviation.
-        return (self.filter_results.standardized_forecasts_error.squeeze() *
-                self.filter_results.obs_cov[0, 0]**0.5)
+        return (self.filter_results.standardized_forecasts_error[0] *
+                self.scale**0.5)
 
     @cache_readonly
     def cusum(self):
@@ -305,7 +365,6 @@ class RecursiveLSResults(MLEResults):
             An array of length `nobs - k_exog` holding the
             CUSUM statistics.
 
-
         Notes
         -----
         The CUSUM statistic takes the form:
@@ -328,16 +387,15 @@ class RecursiveLSResults(MLEResults):
 
         References
         ----------
-        .. [1] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
+        .. [*] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
            "Techniques for Testing the Constancy of
            Regression Relationships over Time."
            Journal of the Royal Statistical Society.
            Series B (Methodological) 37 (2): 149-92.
-
         """
-        llb = self.loglikelihood_burn
-        return (np.cumsum(self.resid_recursive[self.loglikelihood_burn:]) /
-                np.std(self.resid_recursive[llb:], ddof=1))
+        d = max(self.nobs_diffuse, self.loglikelihood_burn)
+        return (np.cumsum(self.resid_recursive[d:]) /
+                np.std(self.resid_recursive[d:], ddof=1))
 
     @cache_readonly
     def cusum_squares(self):
@@ -366,17 +424,116 @@ class RecursiveLSResults(MLEResults):
 
         References
         ----------
-        .. [1] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
+        .. [*] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
            "Techniques for Testing the Constancy of
            Regression Relationships over Time."
            Journal of the Royal Statistical Society.
            Series B (Methodological) 37 (2): 149-92.
-
         """
-        numer = np.cumsum(self.resid_recursive[self.loglikelihood_burn:]**2)
+        d = max(self.nobs_diffuse, self.loglikelihood_burn)
+        numer = np.cumsum(self.resid_recursive[d:]**2)
         denom = numer[-1]
         return numer / denom
 
+    @cache_readonly
+    def llf_recursive_obs(self):
+        """
+        (float) Loglikelihood at observation, computed from recursive residuals
+        """
+        from scipy.stats import norm
+        return np.log(norm.pdf(self.resid_recursive, loc=0,
+                               scale=self.scale**0.5))
+
+    @cache_readonly
+    def llf_recursive(self):
+        """
+        (float) Loglikelihood defined by recursive residuals, equivalent to OLS
+        """
+        return np.sum(self.llf_recursive_obs)
+
+    @cache_readonly
+    def ssr(self):
+        """ssr"""
+        d = max(self.nobs_diffuse, self.loglikelihood_burn)
+        return (self.nobs - d) * self.filter_results.obs_cov[0, 0, 0]
+
+    @cache_readonly
+    def centered_tss(self):
+        """Centered tss"""
+        return np.sum((self.filter_results.endog[0] -
+                       np.mean(self.filter_results.endog))**2)
+
+    @cache_readonly
+    def uncentered_tss(self):
+        """uncentered tss"""
+        return np.sum((self.filter_results.endog[0])**2)
+
+    @cache_readonly
+    def ess(self):
+        """ess"""
+        if self.k_constant:
+            return self.centered_tss - self.ssr
+        else:
+            return self.uncentered_tss - self.ssr
+
+    @cache_readonly
+    def rsquared(self):
+        """rsquared"""
+        if self.k_constant:
+            return 1 - self.ssr / self.centered_tss
+        else:
+            return 1 - self.ssr / self.uncentered_tss
+
+    @cache_readonly
+    def mse_model(self):
+        """mse_model"""
+        return self.ess / self.df_model
+
+    @cache_readonly
+    def mse_resid(self):
+        """mse_resid"""
+        return self.ssr / self.df_resid
+
+    @cache_readonly
+    def mse_total(self):
+        """mse_total"""
+        if self.k_constant:
+            return self.centered_tss / (self.df_resid + self.df_model)
+        else:
+            return self.uncentered_tss / (self.df_resid + self.df_model)
+
+    @Appender(MLEResults.get_prediction.__doc__)
+    def get_prediction(self, start=None, end=None, dynamic=False,
+                       index=None, **kwargs):
+        # Note: need to override this, because we currently do not support
+        # dynamic prediction or forecasts when there are constraints.
+        if start is None:
+            start = self.model._index[0]
+
+        # Handle start, end, dynamic
+        start, end, out_of_sample, prediction_index = (
+            self.model._get_prediction_index(start, end, index))
+
+        # Handle `dynamic`
+        if isinstance(dynamic, (bytes, str)):
+            dynamic, _, _ = self.model._get_index_loc(dynamic)
+
+        if self.model._r_matrix is not None and (out_of_sample or dynamic):
+            raise NotImplementedError('Cannot yet perform out-of-sample or'
+                                      ' dynamic prediction in models with'
+                                      ' constraints.')
+
+        # Perform the prediction
+        # This is a (k_endog x npredictions) array; do not want to squeeze in
+        # case of npredictions = 1
+        prediction_results = self.filter_results.predict(
+            start, end + out_of_sample + 1, dynamic, **kwargs)
+
+        # Return a new mlemodel.PredictionResults object
+        res_obj = PredictionResults(self, prediction_results,
+                                    row_labels=prediction_index)
+        return PredictionResultsWrapper(res_obj)
+
     def plot_recursive_coefficient(self, variables=0, alpha=0.05,
                                    legend_loc='upper left', fig=None,
                                    figsize=None):
@@ -385,15 +542,15 @@ class RecursiveLSResults(MLEResults):
 
         Parameters
         ----------
-        variables : int or str or iterable of int or string, optional
+        variables : {int, str, list[int], list[str]}, optional
             Integer index or string name of the variable whose coefficient will
             be plotted. Can also be an iterable of integers or strings. Default
             is the first variable.
         alpha : float, optional
             The confidence intervals for the coefficient are (1 - alpha) %
-        legend_loc : string, optional
+        legend_loc : str, optional
             The location of the legend in the plot. Default is upper left.
-        fig : Matplotlib Figure instance, optional
+        fig : Figure, optional
             If given, subplots are created in this figure instead of in a new
             figure. Note that the grid will be created in the provided
             figure using `fig.add_subplot()`.
@@ -432,11 +589,11 @@ class RecursiveLSResults(MLEResults):
                 dates = self.data.dates._mpl_repr()
             else:
                 dates = np.arange(self.nobs)
-            llb = self.loglikelihood_burn
+            d = max(self.nobs_diffuse, self.loglikelihood_burn)
 
             # Plot the coefficient
             coef = self.recursive_coefficients
-            ax.plot(dates[llb:], coef.filtered[variable, llb:],
+            ax.plot(dates[d:], coef.filtered[variable, d:],
                     label='Recursive estimates: %s' % exog_names[variable])
 
             # Legend
@@ -453,7 +610,7 @@ class RecursiveLSResults(MLEResults):
                 ci_upper = (
                     coef.filtered[variable] + critical_value * std_errors)
                 ci_poly = ax.fill_between(
-                    dates[llb:], ci_lower[llb:], ci_upper[llb:], alpha=0.2
+                    dates[d:], ci_lower[d:], ci_upper[d:], alpha=0.2
                 )
                 ci_label = ('$%.3g \\%%$ confidence interval'
                             % ((1 - alpha)*100))
@@ -461,7 +618,7 @@ class RecursiveLSResults(MLEResults):
                 # Only add CI to legend for the first plot
                 if i == 0:
                     # Proxy artist for fill_between legend entry
-                    # See http://matplotlib.org/1.3.1/users/legend_guide.html
+                    # See https://matplotlib.org/1.3.1/users/legend_guide.html
                     p = plt.Rectangle((0, 0), 1, 1,
                                       fc=ci_poly.get_facecolor()[0])
 
@@ -498,7 +655,7 @@ class RecursiveLSResults(MLEResults):
         exactly the same confidence bands (which are produced in cusum6 by
         lw, uw) because they burn the first k_exog + 1 periods instead of the
         first k_exog. If this change is performed
-        (so that `tmp = (self.nobs - llb - 1)**0.5`), then the output here
+        (so that `tmp = (self.nobs - d - 1)**0.5`), then the output here
         matches cusum6.
 
         The cusum6 behavior does not seem to be consistent with
@@ -517,12 +674,14 @@ class RecursiveLSResults(MLEResults):
             raise ValueError('Invalid significance level.')
 
         # Get the points for the significance bound lines
-        llb = self.loglikelihood_burn
-        tmp = (self.nobs - llb - ddof)**0.5
-        upper_line = lambda x: scalar * tmp + 2 * scalar * (x - llb) / tmp
+        d = max(self.nobs_diffuse, self.loglikelihood_burn)
+        tmp = (self.nobs - d - ddof)**0.5
+
+        def upper_line(x):
+            return scalar * tmp + 2 * scalar * (x - d) / tmp
 
         if points is None:
-            points = np.array([llb, self.nobs])
+            points = np.array([d, self.nobs])
         return -upper_line(points), upper_line(points)
 
     def plot_cusum(self, alpha=0.05, legend_loc='upper left',
@@ -534,9 +693,9 @@ class RecursiveLSResults(MLEResults):
         ----------
         alpha : float, optional
             The plotted significance bounds are alpha %.
-        legend_loc : string, optional
+        legend_loc : str, optional
             The location of the legend in the plot. Default is upper left.
-        fig : Matplotlib Figure instance, optional
+        fig : Figure, optional
             If given, subplots are created in this figure instead of in a new
             figure. Note that the grid will be created in the provided
             figure using `fig.add_subplot()`.
@@ -551,16 +710,15 @@ class RecursiveLSResults(MLEResults):
 
         References
         ----------
-        .. [1] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
+        .. [*] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
            "Techniques for Testing the Constancy of
            Regression Relationships over Time."
            Journal of the Royal Statistical Society.
            Series B (Methodological) 37 (2): 149-92.
-
         """
         # Create the plot
         from statsmodels.graphics.utils import _import_mpl, create_mpl_fig
-        plt = _import_mpl()
+        _import_mpl()
         fig = create_mpl_fig(fig, figsize)
         ax = fig.add_subplot(1, 1, 1)
 
@@ -569,17 +727,17 @@ class RecursiveLSResults(MLEResults):
             dates = self.data.dates._mpl_repr()
         else:
             dates = np.arange(self.nobs)
-        llb = self.loglikelihood_burn
+        d = max(self.nobs_diffuse, self.loglikelihood_burn)
 
         # Plot cusum series and reference line
-        ax.plot(dates[llb:], self.cusum, label='CUSUM')
-        ax.hlines(0, dates[llb], dates[-1], color='k', alpha=0.3)
+        ax.plot(dates[d:], self.cusum, label='CUSUM')
+        ax.hlines(0, dates[d], dates[-1], color='k', alpha=0.3)
 
         # Plot significance bounds
         lower_line, upper_line = self._cusum_significance_bounds(alpha)
-        ax.plot([dates[llb], dates[-1]], upper_line, 'k--',
+        ax.plot([dates[d], dates[-1]], upper_line, 'k--',
                 label='%d%% significance' % (alpha * 100))
-        ax.plot([dates[llb], dates[-1]], lower_line, 'k--')
+        ax.plot([dates[d], dates[-1]], lower_line, 'k--')
 
         ax.legend(loc=legend_loc)
 
@@ -602,8 +760,8 @@ class RecursiveLSResults(MLEResults):
         """
         # Get the approximate critical value associated with the significance
         # level
-        llb = self.loglikelihood_burn
-        n = 0.5 * (self.nobs - llb) - 1
+        d = max(self.nobs_diffuse, self.loglikelihood_burn)
+        n = 0.5 * (self.nobs - d) - 1
         try:
             ix = [0.1, 0.05, 0.025, 0.01, 0.005].index(alpha / 2)
         except ValueError:
@@ -613,8 +771,8 @@ class RecursiveLSResults(MLEResults):
 
         # Get the points for the significance bound lines
         if points is None:
-            points = np.array([llb, self.nobs])
-        line = (points - llb) / (self.nobs - llb)
+            points = np.array([d, self.nobs])
+        line = (points - d) / (self.nobs - d)
 
         return line - crit, line + crit
 
@@ -627,9 +785,9 @@ class RecursiveLSResults(MLEResults):
         ----------
         alpha : float, optional
             The plotted significance bounds are alpha %.
-        legend_loc : string, optional
+        legend_loc : str, optional
             The location of the legend in the plot. Default is upper left.
-        fig : Matplotlib Figure instance, optional
+        fig : Figure, optional
             If given, subplots are created in this figure instead of in a new
             figure. Note that the grid will be created in the provided
             figure using `fig.add_subplot()`.
@@ -643,24 +801,23 @@ class RecursiveLSResults(MLEResults):
         statistic moves out of the significance bounds.
 
         Critical values used in creating the significance bounds are computed
-        using the approximate formula of [2]_.
+        using the approximate formula of [1]_.
 
         References
         ----------
-        .. [1] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
+        .. [*] Brown, R. L., J. Durbin, and J. M. Evans. 1975.
            "Techniques for Testing the Constancy of
            Regression Relationships over Time."
            Journal of the Royal Statistical Society.
            Series B (Methodological) 37 (2): 149-92.
-        .. [2] Edgerton, David, and Curt Wells. 1994.
+        .. [1] Edgerton, David, and Curt Wells. 1994.
            "Critical Values for the Cusumsq Statistic
            in Medium and Large Sized Samples."
            Oxford Bulletin of Economics and Statistics 56 (3): 355-65.
-
         """
         # Create the plot
         from statsmodels.graphics.utils import _import_mpl, create_mpl_fig
-        plt = _import_mpl()
+        _import_mpl()
         fig = create_mpl_fig(fig, figsize)
         ax = fig.add_subplot(1, 1, 1)
 
@@ -669,18 +826,18 @@ class RecursiveLSResults(MLEResults):
             dates = self.data.dates._mpl_repr()
         else:
             dates = np.arange(self.nobs)
-        llb = self.loglikelihood_burn
+        d = max(self.nobs_diffuse, self.loglikelihood_burn)
 
         # Plot cusum series and reference line
-        ax.plot(dates[llb:], self.cusum_squares, label='CUSUM of squares')
-        ref_line = (np.arange(llb, self.nobs) - llb) / (self.nobs - llb)
-        ax.plot(dates[llb:], ref_line, 'k', alpha=0.3)
+        ax.plot(dates[d:], self.cusum_squares, label='CUSUM of squares')
+        ref_line = (np.arange(d, self.nobs) - d) / (self.nobs - d)
+        ax.plot(dates[d:], ref_line, 'k', alpha=0.3)
 
         # Plot significance bounds
         lower_line, upper_line = self._cusum_squares_significance_bounds(alpha)
-        ax.plot([dates[llb], dates[-1]], upper_line, 'k--',
+        ax.plot([dates[d], dates[-1]], upper_line, 'k--',
                 label='%d%% significance' % (alpha * 100))
-        ax.plot([dates[llb], dates[-1]], lower_line, 'k--')
+        ax.plot([dates[d], dates[-1]], lower_line, 'k--')
 
         ax.legend(loc=legend_loc)
 
@@ -694,4 +851,5 @@ class RecursiveLSResultsWrapper(MLEResultsWrapper):
     _methods = {}
     _wrap_methods = wrap.union_dicts(MLEResultsWrapper._wrap_methods,
                                      _methods)
-wrap.populate_wrapper(RecursiveLSResultsWrapper, RecursiveLSResults)
+wrap.populate_wrapper(RecursiveLSResultsWrapper,  # noqa:E305
+                      RecursiveLSResults)
