@@ -2,16 +2,17 @@
 Test functions for models.regression
 """
 # TODO: Test for LM
-from statsmodels.compat.python import long, lrange
+from statsmodels.compat.python import lrange
 import warnings
 import pandas
 import numpy as np
-from numpy.testing import (assert_almost_equal, assert_approx_equal, assert_,
+from numpy.testing import (assert_almost_equal, assert_,
                            assert_raises, assert_equal, assert_allclose)
+import pytest
 from scipy.linalg import toeplitz
 from statsmodels.tools.tools import add_constant, categorical
-from statsmodels.compat.numpy import np_matrix_rank
-from statsmodels.regression.linear_model import OLS, WLS, GLS, yule_walker
+from statsmodels.regression.linear_model import (OLS, WLS, GLS, yule_walker,
+                                                 burg)
 from statsmodels.datasets import longley
 from scipy.stats import t as student_t
 
@@ -22,6 +23,11 @@ DECIMAL_1 = 1
 DECIMAL_7 = 7
 DECIMAL_0 = 0
 
+try:
+    import cvxopt  # noqa:F401
+    has_cvxopt = True
+except ImportError:
+    has_cvxopt = False
 
 class CheckRegressionResults(object):
     """
@@ -45,10 +51,10 @@ class CheckRegressionResults(object):
         conf1 = self.res1.conf_int()
         conf2 = self.res2.conf_int()
         for i in range(len(conf1)):
-            assert_approx_equal(conf1[i][0], conf2[i][0],
-                                self.decimal_confidenceintervals)
-            assert_approx_equal(conf1[i][1], conf2[i][1],
-                                self.decimal_confidenceintervals)
+            assert_allclose(conf1[i][0], conf2[i][0],
+                            rtol=10**-self.decimal_confidenceintervals)
+            assert_allclose(conf1[i][1], conf2[i][1],
+                            rtol=10**-self.decimal_confidenceintervals)
 
     decimal_conf_int_subset = DECIMAL_4
     def test_conf_int_subset(self):
@@ -107,7 +113,7 @@ class CheckRegressionResults(object):
 
     decimal_fvalue = DECIMAL_4
     def test_fvalue(self):
-        # didn't change this, not sure it should complain -inf not equal -inf
+        # did not change this, not sure it should complain -inf not equal -inf
         # if not (np.isinf(self.res1.fvalue) and np.isinf(self.res2.fvalue)):
         assert_almost_equal(self.res1.fvalue, self.res2.fvalue,
                             self.decimal_fvalue)
@@ -149,9 +155,9 @@ class CheckRegressionResults(object):
 
 class TestOLS(CheckRegressionResults):
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         from .results.results_regression import Longley
-        data = longley.load()
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         res1 = OLS(data.endog, data.exog).fit()
         res2 = Longley()
@@ -165,7 +171,7 @@ class TestOLS(CheckRegressionResults):
         Q, R = np.linalg.qr(data.exog)
         model_qr.exog_Q, model_qr.exog_R = Q, R
         model_qr.normalized_cov_params = np.linalg.inv(np.dot(R.T, R))
-        model_qr.rank = np_matrix_rank(R)
+        model_qr.rank = np.linalg.matrix_rank(R)
         res_qr2 = model_qr.fit(method="qr")
 
         cls.res_qr = res_qr
@@ -184,23 +190,29 @@ class TestOLS(CheckRegressionResults):
         # DECIMAL_4 places for the last place.
         assert_almost_equal(self.res1.HC0_se[:-1],
                             self.res2.HC0_se[:-1], DECIMAL_4)
-        assert_approx_equal(np.round(self.res1.HC0_se[-1]),
-                            self.res2.HC0_se[-1])
+        assert_allclose(np.round(self.res1.HC0_se[-1]),
+                        self.res2.HC0_se[-1])
 
     def test_HC1_errors(self):
         assert_almost_equal(self.res1.HC1_se[:-1],
                             self.res2.HC1_se[:-1], DECIMAL_4)
-        assert_approx_equal(self.res1.HC1_se[-1], self.res2.HC1_se[-1])
+        # Note: tolerance is tight; rtol=3e-7 fails while 4e-7 passes
+        assert_allclose(self.res1.HC1_se[-1], self.res2.HC1_se[-1],
+                        rtol=4e-7)
 
     def test_HC2_errors(self):
         assert_almost_equal(self.res1.HC2_se[:-1],
                             self.res2.HC2_se[:-1], DECIMAL_4)
-        assert_approx_equal(self.res1.HC2_se[-1], self.res2.HC2_se[-1])
+        # Note: tolerance is tight; rtol=4e-7 fails while 5e-7 passes
+        assert_allclose(self.res1.HC2_se[-1], self.res2.HC2_se[-1],
+                        rtol=5e-7)
 
     def test_HC3_errors(self):
         assert_almost_equal(self.res1.HC3_se[:-1],
                             self.res2.HC3_se[:-1], DECIMAL_4)
-        assert_approx_equal(self.res1.HC3_se[-1], self.res2.HC3_se[-1])
+        # Note: tolerance is tight; rtol=1e-7 fails while 1.5e-7 passes
+        assert_allclose(self.res1.HC3_se[-1], self.res2.HC3_se[-1],
+                        rtol=1.5e-7)
 
     def test_qr_params(self):
         assert_almost_equal(self.res1.params,
@@ -213,7 +225,7 @@ class TestOLS(CheckRegressionResults):
                             self.res_qr.normalized_cov_params, 5)
 
     def test_missing(self):
-        data = longley.load()
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         data.endog[[3, 7, 14]] = np.nan
         mod = OLS(data.endog, data.exog, missing='drop')
@@ -251,9 +263,9 @@ class TestOLS(CheckRegressionResults):
 
 class TestRTO(CheckRegressionResults):
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         from .results.results_regression import LongleyRTO
-        data = longley.load()
+        data = longley.load(as_pandas=False)
         res1 = OLS(data.endog, data.exog).fit()
         res2 = LongleyRTO()
         res2.wresid = res1.wresid  # workaround hack
@@ -268,8 +280,8 @@ class TestFtest(object):
     Tests f_test vs. RegressionResults
     """
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         cls.res1 = OLS(data.endog, data.exog).fit()
         R = np.identity(7)[:-1, :]
@@ -296,8 +308,8 @@ class TestFTest2(object):
     Ftest1 is from statsmodels.  Results are from Rpy using R's car library.
     """
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         res1 = OLS(data.endog, data.exog).fit()
         R2 = [[0, 1, -1, 0, 0, 0, 0], [0, 0, 0, 0, 1, -1, 0]]
@@ -313,7 +325,7 @@ class TestFTest2(object):
 
     def test_pvalue(self):
         assert_almost_equal(self.Ftest1.pvalue, 0.0056052885317493459,
-                DECIMAL_4)
+                            DECIMAL_4)
 
     def test_df_denom(self):
         assert_equal(self.Ftest1.df_denom, 9)
@@ -328,8 +340,8 @@ class TestFtestQ(object):
     made up.  Test values taken from Stata.
     """
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         res1 = OLS(data.endog, data.exog).fit()
         R = np.array([[0, 1, 1, 0, 0, 0, 0],
@@ -356,11 +368,10 @@ class TestTtest(object):
     """
     Test individual t-tests.  Ie., are the coefficients significantly
     different than zero.
-
         """
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         cls.res1 = OLS(data.endog, data.exog).fit()
         R = np.identity(7)
@@ -396,10 +407,10 @@ class TestTtest2(object):
     Results from RPy using 'car' package.
     """
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         R = np.zeros(7)
         R[4:6] = [1, -1]
-        data = longley.load()
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         res1 = OLS(data.endog, data.exog).fit()
         cls.Ttest1 = res1.t_test(R)
@@ -426,10 +437,10 @@ class TestGLS(object):
     These test results were obtained by replication with R.
     """
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         from .results.results_regression import LongleyGls
 
-        data = longley.load()
+        data = longley.load(as_pandas=False)
         exog = add_constant(np.column_stack(
             (data.exog[:, 1], data.exog[:, 4])), prepend=False)
         tmp_results = OLS(data.endog, exog).fit()
@@ -446,10 +457,12 @@ class TestGLS(object):
         cls.endog = data.endog
 
     def test_aic(self):
-        assert_approx_equal(self.res1.aic+2, self.res2.aic, 3)
+        # Note: tolerance is tight; rtol=3e-3 fails while 4e-3 passes
+        assert_allclose(self.res1.aic+2, self.res2.aic, rtol=4e-3)
 
     def test_bic(self):
-        assert_approx_equal(self.res1.bic, self.res2.bic, 2)
+        # Note: tolerance is tight; rtol=1e-2 fails while 1.5e-2 passes
+        assert_allclose(self.res1.bic, self.res2.bic, rtol=1.5e-2)
 
     def test_loglike(self):
         assert_almost_equal(self.res1.llf, self.res2.llf, DECIMAL_0)
@@ -484,13 +497,14 @@ class TestGLS(object):
         assert_equal(mod.exog.shape[0], 13)
         assert_equal(mod.sigma.shape, (13, 13))
 
+
 class TestGLS_alt_sigma(CheckRegressionResults):
     """
     Test that GLS with no argument is equivalent to OLS.
     """
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         ols_res = OLS(data.endog, data.exog).fit()
         gls_res = GLS(data.endog, data.exog).fit()
@@ -512,6 +526,7 @@ class TestGLS_alt_sigma(CheckRegressionResults):
         assert_raises(ValueError, GLS, self.endog, self.exog,
                       sigma=np.ones((n-1, n-1)))
 
+# FIXME: do not leave commented-out, use or move/remove
 #    def check_confidenceintervals(self, conf1, conf2):
 #        assert_almost_equal(conf1, conf2, DECIMAL_4)
 
@@ -519,7 +534,7 @@ class TestGLS_alt_sigma(CheckRegressionResults):
 class TestLM(object):
 
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         # TODO: Test HAC method
         X = np.random.randn(100, 3)
         b = np.ones((3, 1))
@@ -602,8 +617,8 @@ class TestLM(object):
 class TestOLS_GLS_WLS_equivalence(object):
 
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         y = data.endog
         X = data.exog
@@ -648,8 +663,8 @@ class TestGLS_WLS_equivalence(TestOLS_GLS_WLS_equivalence):
     # reuse test methods
 
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         y = data.endog
         X = data.exog
@@ -663,20 +678,11 @@ class TestGLS_WLS_equivalence(TestOLS_GLS_WLS_equivalence):
         cls.results.append(GLS(y, X, 100 * w_inv).fit())
         cls.results.append(GLS(y, X, np.diag(0.1 * w_inv)).fit())
 
-    def test_rsquared(self):
-        # TODO: WLS rsquared is ok, GLS might have wrong centered_tss
-        # We only check that WLS and GLS rsquared is invariant to scaling
-        # WLS and GLS have different rsquared
-        assert_almost_equal(self.results[1].rsquared, self.results[0].rsquared,
-                            DECIMAL_7)
-        assert_almost_equal(self.results[3].rsquared, self.results[2].rsquared,
-                            DECIMAL_7)
-
 
 class TestNonFit(object):
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         cls.endog = data.endog
         cls.exog = data.exog
@@ -684,12 +690,12 @@ class TestNonFit(object):
 
     def test_df_resid(self):
         df_resid = self.endog.shape[0] - self.exog.shape[1]
-        assert_equal(self.ols_model.df_resid, long(9))
+        assert_equal(self.ols_model.df_resid, 9)
 
 
 class TestWLS_CornerCases(object):
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         cls.exog = np.ones((1,))
         cls.endog = np.ones((1,))
         weights = 1
@@ -703,10 +709,11 @@ class TestWLS_CornerCases(object):
 class TestWLSExogWeights(CheckRegressionResults):
     # Test WLS with Greene's credit card data
     # reg avgexp age income incomesq ownrent [aw=1/incomesq]
-    def __init__(self):
+    @classmethod
+    def setup_class(cls):
         from .results.results_regression import CCardWLS
         from statsmodels.datasets.ccard import load
-        dta = load()
+        dta = load(as_pandas=False)
 
         dta.exog = add_constant(dta.exog, prepend=False)
         nobs = 72.
@@ -715,15 +722,15 @@ class TestWLSExogWeights(CheckRegressionResults):
         # for comparison with stata analytic weights
         scaled_weights = ((weights * nobs) / weights.sum())
 
-        self.res1 = WLS(dta.endog, dta.exog, weights=scaled_weights).fit()
-        self.res2 = CCardWLS()
-        self.res2.wresid = scaled_weights ** .5 * self.res2.resid
+        cls.res1 = WLS(dta.endog, dta.exog, weights=scaled_weights).fit()
+        cls.res2 = CCardWLS()
+        cls.res2.wresid = scaled_weights ** .5 * cls.res2.resid
 
         # correction because we use different definition for loglike/llf
-        corr_ic = 2 * (self.res1.llf - self.res2.llf)
-        self.res2.aic -= corr_ic
-        self.res2.bic -= corr_ic
-        self.res2.llf += 0.5 * np.sum(np.log(self.res1.model.weights))
+        corr_ic = 2 * (cls.res1.llf - cls.res2.llf)
+        cls.res2.aic -= corr_ic
+        cls.res2.bic -= corr_ic
+        cls.res2.llf += 0.5 * np.sum(np.log(cls.res1.model.weights))
 
 
 def test_wls_example():
@@ -754,9 +761,9 @@ def test_wls_tss():
 
 class TestWLSScalarVsArray(CheckRegressionResults):
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         from statsmodels.datasets.longley import load
-        dta = load()
+        dta = load(as_pandas=False)
         dta.exog = add_constant(dta.exog, prepend=True)
         wls_scalar = WLS(dta.endog, dta.exog, weights=1./3).fit()
         weights = [1/3.] * len(dta.endog)
@@ -764,21 +771,22 @@ class TestWLSScalarVsArray(CheckRegressionResults):
         cls.res1 = wls_scalar
         cls.res2 = wls_array
 
-#class TestWLS_GLS(CheckRegressionResults):
-#    @classmethod
-#    def setupClass(cls):
-#        from statsmodels.datasets.ccard import load
-#        data = load()
-#        cls.res1 = WLS(data.endog, data.exog, weights = 1/data.exog[:,2]).fit()
-#        cls.res2 = GLS(data.endog, data.exog, sigma = data.exog[:,2]).fit()
-#
-#    def check_confidenceintervals(self, conf1, conf2):
-#        assert_almost_equal(conf1, conf2(), DECIMAL_4)
+
+class TestWLS_GLS(CheckRegressionResults):
+    @classmethod
+    def setup_class(cls):
+        from statsmodels.datasets.ccard import load
+        data = load(as_pandas=False)
+        cls.res1 = WLS(data.endog, data.exog, weights=1/data.exog[:, 2]).fit()
+        cls.res2 = GLS(data.endog, data.exog, sigma=data.exog[:,2 ]).fit()
+
+    def check_confidenceintervals(self, conf1, conf2):  # FIXME: never called
+        assert_almost_equal(conf1, conf2(), DECIMAL_4)
 
 
 def test_wls_missing():
     from statsmodels.datasets.ccard import load
-    data = load()
+    data = load(as_pandas=False)
     endog = data.endog
     endog[[10, 25]] = np.nan
     mod = WLS(data.endog, data.exog, weights=1 / data.exog[:, 2],
@@ -790,32 +798,34 @@ def test_wls_missing():
 
 class TestWLS_OLS(CheckRegressionResults):
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         cls.res1 = OLS(data.endog, data.exog).fit()
         cls.res2 = WLS(data.endog, data.exog).fit()
 
-    def check_confidenceintervals(self, conf1, conf2):
+    def check_confidenceintervals(self, conf1, conf2):  # FIXME: never called
         assert_almost_equal(conf1, conf2(), DECIMAL_4)
 
 
 class TestGLS_OLS(CheckRegressionResults):
     @classmethod
-    def setupClass(cls):
-        data = longley.load()
+    def setup_class(cls):
+        data = longley.load(as_pandas=False)
         data.exog = add_constant(data.exog, prepend=False)
         cls.res1 = GLS(data.endog, data.exog).fit()
         cls.res2 = OLS(data.endog, data.exog).fit()
 
-    def check_confidenceintervals(self, conf1, conf2):
+    def check_confidenceintervals(self, conf1, conf2):  # FIXME: never called
         assert_almost_equal(conf1, conf2(), DECIMAL_4)
 
+
+# FIXME: do not leave this commented-out sitting here
 # TODO: test AR
 # why the two-stage in AR?
-# class test_ar(object):
+# class TestAR(object):
 #     from statsmodels.datasets.sunspots import load
-#     data = load()
+#     data = load(as_pandas=False)
 #     model = AR(data.endog, rho=4).fit()
 #     R_res = RModel(data.endog, aic="FALSE", order_max=4)#
 
@@ -830,9 +840,9 @@ class TestGLS_OLS(CheckRegressionResults):
 
 class TestYuleWalker(object):
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         from statsmodels.datasets.sunspots import load
-        data = load()
+        data = load(as_pandas=False)
         cls.rho, cls.sigma = yule_walker(data.endog, order=4,
                                          method="mle")
         cls.R_params = [1.2831003105694765, -0.45240924374091945,
@@ -844,7 +854,7 @@ class TestYuleWalker(object):
 
 class TestDataDimensions(CheckRegressionResults):
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
         np.random.seed(54321)
         cls.endog_n_ = np.random.uniform(0, 20, size=30)
         cls.endog_n_one = cls.endog_n_[:, None]
@@ -860,13 +870,14 @@ class TestDataDimensions(CheckRegressionResults):
         cls.mod2.df_model += 1
         cls.res2 = cls.mod2.fit()
 
-    def check_confidenceintervals(self, conf1, conf2):
+    def check_confidenceintervals(self, conf1, conf2):  # FIXME: never called
         assert_almost_equal(conf1, conf2(), DECIMAL_4)
 
 
 class TestGLS_large_data(TestDataDimensions):
     @classmethod
-    def setupClass(cls):
+    def setup_class(cls):
+        super(TestGLS_large_data, cls).setup_class()
         nobs = 1000
         y = np.random.randn(nobs, 1)
         X = np.random.randn(nobs, 20)
@@ -889,8 +900,8 @@ class TestGLS_large_data(TestDataDimensions):
 
 class TestNxNx(TestDataDimensions):
     @classmethod
-    def setupClass(cls):
-        super(TestNxNx, cls).setupClass()
+    def setup_class(cls):
+        super(TestNxNx, cls).setup_class()
         cls.mod2 = OLS(cls.endog_n_, cls.exog_n_)
         cls.mod2.df_model += 1
         cls.res2 = cls.mod2.fit()
@@ -898,8 +909,8 @@ class TestNxNx(TestDataDimensions):
 
 class TestNxOneNx(TestDataDimensions):
     @classmethod
-    def setupClass(cls):
-        super(TestNxOneNx, cls).setupClass()
+    def setup_class(cls):
+        super(TestNxOneNx, cls).setup_class()
         cls.mod2 = OLS(cls.endog_n_one, cls.exog_n_)
         cls.mod2.df_model += 1
         cls.res2 = cls.mod2.fit()
@@ -907,8 +918,8 @@ class TestNxOneNx(TestDataDimensions):
 
 class TestNxNxOne(TestDataDimensions):
     @classmethod
-    def setupClass(cls):
-        super(TestNxNxOne, cls).setupClass()
+    def setup_class(cls):
+        super(TestNxNxOne, cls).setup_class()
         cls.mod2 = OLS(cls.endog_n_, cls.exog_n_one)
         cls.mod2.df_model += 1
         cls.res2 = cls.mod2.fit()
@@ -925,14 +936,26 @@ def test_const_indicator():
     X = np.random.randint(0, 3, size=30)
     X = categorical(X, drop=True)
     y = np.dot(X, [1., 2., 3.]) + np.random.normal(size=30)
-    modc = OLS(y, add_constant(X[:, 1:], prepend=True)).fit()
-    mod = OLS(y, X, hasconst=True).fit()
-    assert_almost_equal(modc.rsquared, mod.rsquared, 12)
+    resc = OLS(y, add_constant(X[:, 1:], prepend=True)).fit()
+    res = OLS(y, X, hasconst=True).fit()
+    assert_almost_equal(resc.rsquared, res.rsquared, 12)
+    assert res.model.data.k_constant == 1
+    assert resc.model.data.k_constant == 1
 
 
-def test_706():
-    # make sure one regressor pandas Series gets passed to DataFrame
-    # for conf_int.
+def test_fvalue_const_only():
+    np.random.seed(12345)
+    x = np.random.randint(0, 3, size=30)
+    x = categorical(x, drop=True)
+    x[:, 0] = 1
+    y = np.dot(x, [1., 2., 3.]) + np.random.normal(size=30)
+    res = OLS(y, x, hasconst=True).fit(cov_type='HC1')
+    assert not np.isnan(res.fvalue)
+
+
+def test_conf_int_single_regressor():
+    # GH#706 single-regressor model (i.e. no intercept) with 1D exog
+    # should get passed to DataFrame for conf_int
     y = pandas.Series(np.random.randn(10))
     x = pandas.Series(np.ones(10))
     res = OLS(y, x).fit()
@@ -941,20 +964,20 @@ def test_706():
     np.testing.assert_(isinstance(conf_int, pandas.DataFrame))
 
 
-def test_summary():
-    # test 734
+def test_summary_as_latex():
+    # GH#734
     import re
     dta = longley.load_pandas()
     X = dta.exog
     X["constant"] = 1
     y = dta.endog
-    with warnings.catch_warnings(record=True):
-        res = OLS(y, X).fit()
+    res = OLS(y, X).fit()
+    with pytest.warns(UserWarning):
         table = res.summary().as_latex()
     # replace the date and time
-    table = re.sub("(?<=\n\\\\textbf\{Date:\}             &).+?&",
+    table = re.sub("(?<=\n\\\\textbf\\{Date:\\}             &).+?&",
                    " Sun, 07 Apr 2013 &", table)
-    table = re.sub("(?<=\n\\\\textbf\{Time:\}             &).+?&",
+    table = re.sub("(?<=\n\\\\textbf\\{Time:\\}             &).+?&",
                    "     13:46:07     &", table)
 
     expected = """\\begin{center}
@@ -971,7 +994,7 @@ def test_summary():
 \\bottomrule
 \\end{tabular}
 \\begin{tabular}{lcccccc}
-                  & \\textbf{coef} & \\textbf{std err} & \\textbf{t} & \\textbf{P$>$$|$t$|$} & \\textbf{[0.025} & \\textbf{0.975]}  \\\\
+                  & \\textbf{coef} & \\textbf{std err} & \\textbf{t} & \\textbf{P$> |$t$|$} & \\textbf{[0.025} & \\textbf{0.975]}  \\\\
 \\midrule
 \\textbf{GNPDEFL}  &      15.0619  &       84.915     &     0.177  &         0.863        &     -177.029    &      207.153     \\\\
 \\textbf{GNP}      &      -0.0358  &        0.033     &    -1.070  &         0.313        &       -0.112    &        0.040     \\\\
@@ -990,14 +1013,18 @@ def test_summary():
 \\bottomrule
 \\end{tabular}
 %\\caption{OLS Regression Results}
-\\end{center}"""
+\\end{center}
+
+Warnings: \\newline
+ [1] Standard Errors assume that the covariance matrix of the errors is correctly specified. \\newline
+ [2] The condition number is large, 4.86e+09. This might indicate that there are \\newline
+ strong multicollinearity or other numerical problems."""
     assert_equal(table, expected)
 
 
 class TestRegularizedFit(object):
 
-    # Make sure there are no issues when there are no selected
-    # variables.
+    # Make sure there are no problems when no variables are selected.
     def test_empty_model(self):
 
         np.random.seed(742)
@@ -1005,15 +1032,15 @@ class TestRegularizedFit(object):
         endog = np.random.normal(size=n)
         exog = np.random.normal(size=(n, 3))
 
-        model = OLS(endog, exog)
-        result = model.fit_regularized(alpha=1000)
-
-        assert_equal(result.params, 0.)
+        for cls in OLS, WLS, GLS:
+            model = cls(endog, exog)
+            result = model.fit_regularized(alpha=1000)
+            assert_equal(result.params, 0.)
 
     def test_regularized(self):
 
         import os
-        from . import glmnet_r_results
+        from .results import glmnet_r_results
 
         cur_dir = os.path.dirname(os.path.abspath(__file__))
         data = np.loadtxt(os.path.join(cur_dir, "results", "lasso_data.csv"),
@@ -1039,16 +1066,46 @@ class TestRegularizedFit(object):
             exog = exog - exog.mean(0)
             exog /= exog.std(0, ddof=1)
 
-            mod = OLS(endog, exog)
-            rslt = mod.fit_regularized(L1_wt=L1_wt, alpha=lam)
-            assert_almost_equal(rslt.params, params, decimal=3)
+            for cls in OLS, WLS, GLS:
+                mod = cls(endog, exog)
+                rslt = mod.fit_regularized(L1_wt=L1_wt, alpha=lam)
+                assert_almost_equal(rslt.params, params, decimal=3)
 
-            # Smoke test for summary
-            rslt.summary()
+                # Smoke test for profile likelihood
+                mod.fit_regularized(L1_wt=L1_wt, alpha=lam,
+                                    profile_scale=True)
 
-            # Smoke test for profile likeihood
-            mod.fit_regularized(L1_wt=L1_wt, alpha=lam,
-                                profile_scale=True)
+    def test_regularized_weights(self):
+
+        np.random.seed(1432)
+        exog1 = np.random.normal(size=(100, 3))
+        endog1 = exog1[:, 0] + exog1[:, 1] + np.random.normal(size=100)
+        exog2 = np.random.normal(size=(100, 3))
+        endog2 = exog2[:, 0] + exog2[:, 1] + np.random.normal(size=100)
+
+        exog_a = np.vstack((exog1, exog1, exog2))
+        endog_a = np.concatenate((endog1, endog1, endog2))
+
+        # Should be equivalent to exog_a, endog_a.
+        exog_b = np.vstack((exog1, exog2))
+        endog_b = np.concatenate((endog1, endog2))
+        wgts = np.ones(200)
+        wgts[0:100] = 2
+        sigma = np.diag(1/wgts)
+
+        for L1_wt in 0, 0.5, 1:
+            for alpha in 0, 1:
+                mod1 = OLS(endog_a, exog_a)
+                rslt1 = mod1.fit_regularized(L1_wt=L1_wt, alpha=alpha)
+
+                mod2 = WLS(endog_b, exog_b, weights=wgts)
+                rslt2 = mod2.fit_regularized(L1_wt=L1_wt, alpha=alpha)
+
+                mod3 = GLS(endog_b, exog_b, sigma=sigma)
+                rslt3 = mod3.fit_regularized(L1_wt=L1_wt, alpha=alpha)
+
+                assert_almost_equal(rslt1.params, rslt2.params, decimal=3)
+                assert_almost_equal(rslt1.params, rslt3.params, decimal=3)
 
 
 def test_formula_missing_cat():
@@ -1059,7 +1116,7 @@ def test_formula_missing_cat():
     from patsy import PatsyError
 
     dta = sm.datasets.grunfeld.load_pandas().data
-    dta.ix[0, 'firm'] = np.nan
+    dta.loc[dta.index[0], 'firm'] = np.nan
 
     mod = ols(formula='value ~ invest + capital + firm + year',
               data=dta.dropna())
@@ -1079,9 +1136,9 @@ def test_missing_formula_predict():
     # see 2171
     nsample = 30
 
-    data = pandas.DataFrame({'x': np.linspace(0, 10, nsample)})
-    null = pandas.DataFrame({'x': np.array([np.nan])})
-    data = pandas.concat([data, null])
+    data = np.linspace(0, 10, nsample)
+    null = np.array([np.nan])
+    data = pandas.DataFrame({'x': np.concatenate((data, null))})
     beta = np.array([1, 0.1])
     e = np.random.normal(size=nsample+1)
     data['y'] = beta[0] + beta[1] * data['x'] + e
@@ -1091,6 +1148,7 @@ def test_missing_formula_predict():
 
 
 def test_fvalue_implicit_constant():
+    # if constant is implicit, return nan see #2444
     nobs = 100
     np.random.seed(2)
     x = np.random.randn(nobs, 1)
@@ -1110,6 +1168,26 @@ def test_fvalue_implicit_constant():
     res.summary()
 
 
+def test_fvalue_only_constant():
+    # if only constant in model, return nan see #3642
+    nobs = 20
+    np.random.seed(2)
+    x = np.ones(nobs)
+    y = np.random.randn(nobs)
+
+    from statsmodels.regression.linear_model import OLS, WLS
+
+    res = OLS(y, x).fit(cov_type='hac', cov_kwds={'maxlags': 3})
+    assert_(np.isnan(res.fvalue))
+    assert_(np.isnan(res.f_pvalue))
+    res.summary()
+
+    res = WLS(y, x).fit(cov_type='HC1')
+    assert_(np.isnan(res.fvalue))
+    assert_(np.isnan(res.f_pvalue))
+    res.summary()
+
+
 def test_ridge():
     n = 100
     p = 5
@@ -1117,12 +1195,19 @@ def test_ridge():
     xmat = np.random.normal(size=(n, p))
     yvec = xmat.sum(1) + np.random.normal(size=n)
 
-    for alpha in [1., np.ones(p), 10, 10*np.ones(p)]:
-        model1 = OLS(yvec, xmat)
-        result1 = model1._fit_ridge(alpha=1.)
-        model2 = OLS(yvec, xmat)
-        result2 = model2.fit_regularized(alpha=1., L1_wt=0)
-        assert_allclose(result1.params, result2.params)
+    v = np.ones(p)
+    v[0] = 0
+
+    for a in (0, 1, 10):
+        for alpha in (a, a*np.ones(p), a*v):
+            model1 = OLS(yvec, xmat)
+            result1 = model1._fit_ridge(alpha=alpha)
+            model2 = OLS(yvec, xmat)
+            result2 = model2.fit_regularized(alpha=alpha, L1_wt=0)
+            assert_allclose(result1.params, result2.params)
+            model3 = OLS(yvec, xmat)
+            result3 = model3.fit_regularized(alpha=alpha, L1_wt=1e-10)
+            assert_allclose(result1.params, result3.params)
 
     fv1 = result1.fittedvalues
     fv2 = np.dot(xmat, result1.params)
@@ -1134,15 +1219,38 @@ def test_regularized_refit():
     p = 5
     np.random.seed(3132)
     xmat = np.random.normal(size=(n, p))
-    yvec = xmat.sum(1) + np.random.normal(size=n)
+    # covariates 0 and 2 matter
+    yvec = xmat[:, 0] + xmat[:, 2] + np.random.normal(size=n)
     model1 = OLS(yvec, xmat)
     result1 = model1.fit_regularized(alpha=2., L1_wt=0.5, refit=True)
-    model2 = OLS(yvec, xmat)
-    result2 = model2.fit_regularized(alpha=2., L1_wt=0.5, refit=True)
-    assert_allclose(result1.params, result2.params)
-    assert_allclose(result1.bse, result2.bse)
+    model2 = OLS(yvec, xmat[:, [0, 2]])
+    result2 = model2.fit()
+    ii = [0, 2]
+    assert_allclose(result1.params[ii], result2.params)
+    assert_allclose(result1.bse[ii], result2.bse)
 
 
+def test_regularized_predict():
+    n = 100
+    p = 5
+    np.random.seed(3132)
+    xmat = np.random.normal(size=(n, p))
+    yvec = xmat.sum(1) + np.random.normal(size=n)
+    wgt = np.random.uniform(1, 2, n)
+
+    for klass in WLS, GLS:
+        model1 = klass(yvec, xmat,  weights=wgt)
+        result1 = model1.fit_regularized(alpha=2., L1_wt=0.5, refit=True)
+
+        params = result1.params
+        fittedvalues = np.dot(xmat, params)
+        pr = model1.predict(result1.params)
+        assert_allclose(fittedvalues, pr)
+        assert_allclose(result1.fittedvalues, pr)
+
+        pr = result1.predict()
+        assert_allclose(fittedvalues, pr)
+
 def test_regularized_options():
     n = 100
     p = 5
@@ -1157,11 +1265,99 @@ def test_regularized_options():
     assert_allclose(result1.params, result2.params)
 
 
-if __name__ == "__main__":
+def test_burg():
+    rnd = np.random.RandomState(12345)
+    e = rnd.randn(10001)
+    y = e[1:] + 0.5 * e[:-1]
+    # R, ar.burg
+    expected = [
+        [0.3909931],
+        [0.4602607, -0.1771582],
+        [0.47473245, -0.21475602, 0.08168813],
+        [0.4787017, -0.2251910, 0.1047554, -0.0485900],
+        [0.47975462, - 0.22746106, 0.10963527, -0.05896347, 0.02167001]
+    ]
 
-    import nose
-    # run_module_suite()
-    nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
-                   exit=False)
+    for i in range(1, 6):
+        ar, _ = burg(y, i)
+        assert_allclose(ar, expected[i - 1], atol=1e-6)
+        as_nodemean, _ = burg(1 + y, i, False)
+        assert np.all(ar != as_nodemean)
 
-    # nose.runmodule(argv=[__file__,'-vvs','-x'], exit=False) #, '--pdb'
+
+def test_burg_errors():
+    with pytest.raises(ValueError):
+        burg(np.ones((100, 2)))
+    with pytest.raises(ValueError):
+        burg(np.random.randn(100), 0)
+    with pytest.raises(ValueError):
+        burg(np.random.randn(100), 'apple')
+
+
+@pytest.mark.skipif(not has_cvxopt, reason="sqrt_lasso requires cvxopt")
+def test_sqrt_lasso():
+
+    np.random.seed(234923)
+
+    # Based on the example in the Belloni paper
+    n = 100
+    p = 500
+    ii = np.arange(p)
+    cx = 0.5 ** np.abs(np.subtract.outer(ii, ii))
+    cxr = np.linalg.cholesky(cx)
+
+    x = np.dot(np.random.normal(size=(n, p)), cxr.T)
+    b = np.zeros(p)
+    b[0:5] = [1, 1, 1, 1, 1]
+
+    from scipy.stats.distributions import norm
+    alpha = 1.1 * np.sqrt(n) * norm.ppf(1 - 0.05 / (2 * p))
+
+    # Use very low noise level for a unit test
+    y = np.dot(x, b) + 0.25 * np.random.normal(size=n)
+
+    # At low noise levels, the sqrt lasso should be around a
+    # factor of 3 from the oracle without refit, and should
+    # almost equal the oracle with refit.
+    expected_oracle = {False: 3, True: 1}
+
+    # Used for regression testing
+    expected_params = {False: np.r_[0.87397122, 0.96051874, 0.9905915 , 0.93868953, 0.90771773],
+                       True: np.r_[0.95114241, 1.0302987 , 1.01723074, 0.97587343, 0.99846403]}
+
+    for refit in False, True:
+
+        rslt = OLS(y, x).fit_regularized(method="sqrt_lasso", alpha=alpha, refit=refit)
+        err = rslt.params - b
+        numer = np.sqrt(np.dot(err, np.dot(cx, err)))
+
+        oracle = OLS(y, x[:, 0:5]).fit()
+        oracle_err = np.zeros(p)
+        oracle_err[0:5] = oracle.params - b[0:5]
+        denom = np.sqrt(np.dot(oracle_err, np.dot(cx, oracle_err)))
+
+        # Check performance relative to oracle, should be around
+        assert_allclose(numer / denom, expected_oracle[refit],
+             rtol=0.5, atol=0.1)
+
+        # Regression test the parameters
+        assert_allclose(rslt.params[0:5], expected_params[refit],
+                rtol=1e-5, atol=1e-5)
+
+
+def test_bool_regressor(reset_randomstate):
+    exog = np.random.randint(0, 2, size=(100, 2)).astype(bool)
+    endog = np.random.standard_normal(100)
+    bool_res = OLS(endog, exog).fit()
+    res = OLS(endog, exog.astype(np.double)).fit()
+    assert_allclose(bool_res.params, res.params)
+
+
+def test_ols_constant(reset_randomstate):
+    y = np.random.standard_normal((200))
+    x = np.ones((200, 1))
+    res = OLS(y, x).fit()
+    with pytest.warns(None) as recording:
+        assert np.isnan(res.fvalue)
+        assert np.isnan(res.f_pvalue)
+    assert len(recording) == 0
