@@ -11,7 +11,6 @@ effects and random coefficients, and uses OneWayMixed to estimate it.
 
 
 """
-from __future__ import print_function
 import numpy as np
 
 from statsmodels.sandbox.panel.mixed import OneWayMixed, Unit
@@ -115,7 +114,7 @@ if 'ex1' in examples:
     >>> m.cov_random()
     array([[ 0.0348722 , -0.00909159],
            [-0.00909159,  0.26846254]])
-    >>> #note cov_random doesn't subtract mean!
+    >>> #note cov_random does not subtract mean!
     '''
     print('\nchecking the random effects distribution and prediction')
     gamma_re_true = np.array(gamma_re_true)
@@ -138,7 +137,7 @@ if 'ex1' in examples:
     print('rmse_perc (std)', rmse_perc)
     #from numpy.testing import assert_almost_equal
     #assert is for n_units=100 in original example
-    #I changed random number generation, so this won't work anymore
+    #I changed random number generation, so this will not work anymore
     #assert_almost_equal(rmse_perc, [ 34.14783884,  11.6031684 ], decimal=8)
 
     #now returns res
