@@ -6,9 +6,8 @@ import numpy as np
 import statsmodels.api as sm
 import scikits.timeseries as ts
 import scikits.timeseries.lib.plotlib as tplt
-from matplotlib import pyplot as plt
 
-data = sm.datasets.macrodata.load()
+data = sm.datasets.macrodata.load(as_pandas=False)
 data = data.data
 
 
