@@ -7,7 +7,9 @@ Author: Josef Perktold
 """
 import warnings
 
+import pytest
 import numpy as np
+import pandas as pd
 from numpy.testing import (assert_almost_equal, assert_equal, assert_array_less,
                            assert_raises, assert_allclose)
 
@@ -15,35 +17,81 @@ from statsmodels.stats.proportion import (proportion_confint,
                                           multinomial_proportions_confint)
 import statsmodels.stats.proportion as smprop
 from statsmodels.tools.sm_exceptions import HypothesisTestWarning
+from statsmodels.tools.testing import Holder
 
 
-class Holder(object):
-    pass
-
+probci_methods = {'agresti_coull': 'agresti-coull',
+                  'normal': 'asymptotic',
+                  'beta': 'exact',
+                  'wilson': 'wilson',
+                  'jeffreys': 'bayes'
+                  }
 
 
 def test_confint_proportion():
     from .results.results_proportion import res_binom, res_binom_methods
-    methods = {'agresti_coull' : 'agresti-coull',
-               'normal' : 'asymptotic',
-               'beta' : 'exact',
-               'wilson' : 'wilson',
-               'jeffrey' : 'bayes'
-               }
+
 
     for case in res_binom:
         count, nobs = case
-        for method in methods:
-            idx = res_binom_methods.index(methods[method])
+        for method in probci_methods:
+            idx = res_binom_methods.index(probci_methods[method])
             res_low = res_binom[case].ci_low[idx]
             res_upp = res_binom[case].ci_upp[idx]
             if np.isnan(res_low) or np.isnan(res_upp):
                 continue
+            if (count == 0 or count == nobs) and method == 'jeffreys':
+                # maybe a bug or different corner case definition
+                continue
+            if method == 'jeffreys' and nobs == 30:
+                # something is strange in extreme case e.g 0/30 or 1/30
+                continue
             ci = proportion_confint(count, nobs, alpha=0.05, method=method)
-
+            # we impose that confint is in [0, 1]
+            res_low = max(res_low, 0)
+            res_upp = min(res_upp, 1)
             assert_almost_equal(ci, [res_low, res_upp], decimal=6,
                                 err_msg=repr(case) + method)
 
+
+@pytest.mark.parametrize('method', probci_methods)
+def test_confint_proportion_ndim(method):
+    # check that it works with 1-D, 2-D and pandas
+
+    count = np.arange(6).reshape(2, 3)
+    nobs = 10 * np.ones((2, 3))
+
+    count_pd = pd.DataFrame(count)
+    nobs_pd =  pd.DataFrame(nobs)
+
+    ci_arr = proportion_confint(count, nobs, alpha=0.05, method=method)
+    ci_pd = proportion_confint(count_pd, nobs_pd, alpha=0.05,
+                               method=method)
+    assert_allclose(ci_arr, (ci_pd[0].values, ci_pd[1].values), rtol=1e-13)
+    # spot checking one value
+    ci12 = proportion_confint(count[1, 2], nobs[1, 2], alpha=0.05,
+                              method=method)
+    assert_allclose((ci_pd[0].values[1, 2], ci_pd[1].values[1, 2]), ci12,
+                    rtol=1e-13)
+    assert_allclose((ci_arr[0][1, 2], ci_arr[1][1, 2]), ci12, rtol=1e-13)
+
+    # check that lists work as input
+    ci_li = proportion_confint(count.tolist(), nobs.tolist(), alpha=0.05,
+                               method=method)
+    assert_allclose(ci_arr, (ci_li[0], ci_li[1]), rtol=1e-13)
+
+    # check pandas Series, 1-D
+    ci_pds = proportion_confint(count_pd.iloc[0], nobs_pd.iloc[0],
+                                alpha=0.05, method=method)
+    assert_allclose((ci_pds[0].values, ci_pds[1].values),
+                    (ci_pd[0].values[0], ci_pd[1].values[0]), rtol=1e-13)
+
+    # check scalar nobs, verifying one value
+    ci_arr2 = proportion_confint(count, nobs[1, 2], alpha=0.05,
+                                 method=method)
+    assert_allclose((ci_arr2[0][1, 2], ci_arr[1][1, 2]), ci12, rtol=1e-13)
+
+
 def test_samplesize_confidenceinterval_prop():
     #consistency test for samplesize to achieve confidence_interval
     nobs = 20
@@ -79,7 +127,7 @@ def test_multinomial_proportions_errors():
         assert_raises(NotImplementedError, multinomial_proportions_confint,
                       [5] * 50, method=method)
 
-def  test_confint_multinomial_proportions_zeros():
+def test_confint_multinomial_proportions_zeros():
     # test when a count is zero or close to zero
     # values from R MultinomialCI
     ci01 = np.array([
@@ -392,7 +440,7 @@ def test_power_binom_tost():
     power = smprop.power_binom_tost(0.4, 0.6, nobs, p_alt=0.5, alpha=0.05)
     res_power = np.array([ 0.,  0.,  0.,  0.0889,  0.2356,  0.3517,  0.4457,
                            0.6154,  0.6674,  0.7708])
-    # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+    # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
     assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
 def test_power_ztost_prop():
@@ -408,7 +456,7 @@ def test_power_ztost_prop():
 
         res_power = np.array([ 0., 0., 0., 0.0889, 0.2356, 0.4770, 0.5530,
             0.6154,  0.7365,  0.7708])
-        # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+        # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
         assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
         # with critval_continuity correction
@@ -419,7 +467,7 @@ def test_power_ztost_prop():
 
         res_power = np.array([0., 0., 0., 0.0889, 0.2356, 0.3517, 0.4457,
                               0.6154, 0.6674, 0.7708])
-        # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+        # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
         assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
         power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
@@ -429,7 +477,7 @@ def test_power_ztost_prop():
 
         res_power = np.array([0., 0., 0., 0.0889, 0.2356, 0.3517, 0.4457,
                               0.6154, 0.6674, 0.7112])
-        # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+        # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
         assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
 def test_ztost():
@@ -452,59 +500,64 @@ def test_ztost():
 def test_power_ztost_prop_norm():
     # regression test for normal distribution
     # from a rough comparison, the results and variations look reasonable
-    power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
-                                    p_alt=0.5, alpha=0.05, discrete=False,
-                                    dist='norm', variance_prop=0.5,
-                                    continuity=0, critval_continuity=0)[0]
+    with pytest.warns(HypothesisTestWarning):
+        power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
+                                        p_alt=0.5, alpha=0.05, discrete=False,
+                                        dist='norm', variance_prop=0.5,
+                                        continuity=0, critval_continuity=0)[0]
 
-    res_power = np.array([0., 0., 0., 0.11450013,  0.27752006, 0.41495922,
-                          0.52944621,  0.62382638,  0.70092914,  0.76341806])
-    # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+    res_power = np.array([0., 0., 0., 0.11450013, 0.27752006, 0.41495922,
+                          0.52944621, 0.62382638, 0.70092914, 0.76341806])
+    # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
     assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
     # regression test for normal distribution
-    power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
-                                    p_alt=0.5, alpha=0.05, discrete=False,
-                                    dist='norm', variance_prop=0.5,
-                                    continuity=1, critval_continuity=0)[0]
+    with pytest.warns(HypothesisTestWarning):
+        power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
+                                        p_alt=0.5, alpha=0.05, discrete=False,
+                                        dist='norm', variance_prop=0.5,
+                                        continuity=1, critval_continuity=0)[0]
 
-    res_power = np.array([0., 0., 0.02667562,  0.20189793,  0.35099606,
-                          0.47608598,  0.57981118,  0.66496683,  0.73427591,
+    res_power = np.array([0., 0., 0.02667562, 0.20189793, 0.35099606,
+                          0.47608598, 0.57981118, 0.66496683, 0.73427591,
                           0.79026127])
-    # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+    # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
     assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
     # regression test for normal distribution
-    power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
-                                    p_alt=0.5, alpha=0.05, discrete=True,
-                                    dist='norm', variance_prop=0.5,
-                                    continuity=1, critval_continuity=0)[0]
+    with pytest.warns(HypothesisTestWarning):
+        power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
+                                        p_alt=0.5, alpha=0.05, discrete=True,
+                                        dist='norm', variance_prop=0.5,
+                                        continuity=1, critval_continuity=0)[0]
 
-    res_power = np.array([0., 0., 0., 0.08902071,  0.23582284, 0.35192313,
-                          0.55312718,  0.61549537,  0.66743625,  0.77066806])
-    # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+    res_power = np.array([0., 0., 0., 0.08902071, 0.23582284, 0.35192313,
+                          0.55312718, 0.61549537, 0.66743625, 0.77066806])
+    # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
     assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
     # regression test for normal distribution
-    power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
-                                    p_alt=0.5, alpha=0.05, discrete=True,
-                                    dist='norm', variance_prop=0.5,
-                                    continuity=1, critval_continuity=1)[0]
+    with pytest.warns(HypothesisTestWarning):
+        power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
+                                        p_alt=0.5, alpha=0.05, discrete=True,
+                                        dist='norm', variance_prop=0.5,
+                                        continuity=1, critval_continuity=1)[0]
 
-    res_power = np.array([0., 0., 0., 0.08902071,  0.23582284, 0.35192313,
-                          0.44588687,  0.61549537,  0.66743625,  0.71115563])
-    # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+    res_power = np.array([0., 0., 0., 0.08902071, 0.23582284, 0.35192313,
+                          0.44588687, 0.61549537, 0.66743625, 0.71115563])
+    # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
     assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
     # regression test for normal distribution
-    power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
-                                    p_alt=0.5, alpha=0.05, discrete=True,
-                                    dist='norm', variance_prop=None,
-                                    continuity=0, critval_continuity=0)[0]
+    with pytest.warns(HypothesisTestWarning):
+        power = smprop.power_ztost_prop(0.4, 0.6, np.arange(20, 210, 20),
+                                        p_alt=0.5, alpha=0.05, discrete=True,
+                                        dist='norm', variance_prop=None,
+                                        continuity=0, critval_continuity=0)[0]
 
     res_power = np.array([0., 0., 0., 0., 0.15851942, 0.41611758,
-                          0.5010377 ,  0.5708047 ,  0.70328247,  0.74210096])
-    # TODO: I currently don't impose power>=0, i.e np.maximum(power, 0)
+                          0.5010377, 0.5708047, 0.70328247, 0.74210096])
+    # TODO: I currently do not impose power>=0, i.e np.maximum(power, 0)
     assert_almost_equal(np.maximum(power, 0), res_power, decimal=4)
 
 
@@ -521,6 +574,3 @@ def test_proportion_ztests():
     res2 = smprop.proportions_chisquare(np.asarray([15, 10]), np.asarray([20., 20]))
     # test only p-value
     assert_almost_equal(res1[1], res2[1], decimal=13)
-
-if __name__ == '__main__':
-    test_confint_proportion()
